@@ -12,7 +12,6 @@ Notredame, Higgins, Heringa, JMB,302(205-217) 2000
 Last checked against: Version_6.92
 """
 
-import types
 from Bio.Application import _Option, _Switch, _Argument, AbstractCommandline
 
 class TCoffeeCommandline(AbstractCommandline):
@@ -66,13 +65,13 @@ class TCoffeeCommandline(AbstractCommandline):
                     "Default: blosum62mt",
                     0),
            _Option(["-gapopen", "gapopen"], ["input"],
-                    lambda x: isinstance(x, types.IntType),
+                    lambda x: isinstance(x, int),
                     0,
                     "Indicates the penalty applied for opening a gap "
                     "(negative integer)",
                     0),
            _Option(["-gapext", "gapext"], ["input"],
-                    lambda x: isinstance(x, types.IntType),
+                    lambda x: isinstance(x, int),
                     0,
                     "Indicates the penalty applied for extending a "
                     "gap. (negative integer)",
