@@ -64,7 +64,8 @@ class CodonTable:
         e.g.
         >>> import Bio.Data.CodonTable
         >>> print Bio.Data.CodonTable.standard_dna_table
-        >>> print Bio.Data.CodonTable.generic_by_id[1]"""
+        >>> print Bio.Data.CodonTable.generic_by_id[1]
+        """
 
         if self.id:
             answer = "Table %i" % self.id
@@ -120,8 +121,7 @@ def make_back_table(table, default_stop_codon):
     # Do the sort so changes in the hash implementation won't affect
     # the result when one amino acid is coded by more than one codon.
     back_table = {}
-    keys = table.keys() ; keys.sort()
-    for key in keys:
+    for key in sorted(table):
         back_table[table[key]] = key
     back_table[None] = default_stop_codon
     return back_table
@@ -193,7 +193,7 @@ def list_possible_proteins(codon, forward_table, ambiguous_nucleotide_values):
                         # If tripping over a stop codon
                         stops.append(y1+y2+y3)
         if stops:
-            if possible.keys():
+            if possible:
                 raise TranslationError("ambiguous codon '%s' codes " % codon \
                                        + "for both proteins and stop codons")
             # This is a true stop codon - tell the caller about it
@@ -281,12 +281,12 @@ class AmbiguousForwardTable:
         self.ambiguous_protein = ambiguous_protein
 
         inverted = {}
-        for name, val in ambiguous_protein.items():
+        for name, val in ambiguous_protein.iteritems():
             for c in val:
                 x = inverted.get(c, {})
                 x[name] = 1
                 inverted[c] = x
-        for name, val in inverted.items():
+        for name, val in inverted.iteritems():
             inverted[name] = val.keys()
         self._inverted = inverted
         
@@ -344,7 +344,7 @@ class AmbiguousForwardTable:
 
         n = len(possible)
         possible = []
-        for amino, val in ambiguous_possible.items():
+        for amino, val in ambiguous_possible.iteritems():
             if val == n:
                 possible.append(amino)
 
@@ -356,12 +356,14 @@ class AmbiguousForwardTable:
         # All of these are valid, so choose one
         # To be unique, sort by smallet ambiguity then alphabetically
         # Can get this if "X" encodes for everything.
-        def _sort(x, y, table = self.ambiguous_protein):
-            a = cmp(len(table[x]), len(table[y]))
-            if a == 0:
-                return cmp(x, y)
-            return a
-        possible.sort(_sort)
+        #def _sort(x, y, table = self.ambiguous_protein):
+        #    a = cmp(len(table[x]), len(table[y]))
+        #    if a == 0:
+        #        return cmp(x, y)
+        #    return a
+
+        #Sort by key is 2.x and 3.x compatible
+        possible.sort(key=lambda x:(len(self.ambiguous_protein[x]), x))
                           
         x = possible[0]
         self._cache[codon] = x
@@ -387,7 +389,7 @@ def register_ncbi_table(name, alt_name, id,
     # replace all T's with U's for the RNA tables
     rna_table = {}
     generic_table = {}
-    for codon, val in table.items():
+    for codon, val in table.iteritems():
         generic_table[codon] = val
         codon = codon.replace("T", "U")
         generic_table[codon] = val
@@ -854,13 +856,13 @@ register_ncbi_table(name = 'Thraustochytrium Mitochondrial',
 
 
 #Basic sanity test,
-for key, val in generic_by_name.items():
+for key, val in generic_by_name.iteritems():
     assert key in ambiguous_generic_by_name[key].names
-for key, val in generic_by_id.items():
+for key, val in generic_by_id.iteritems():
     assert ambiguous_generic_by_id[key].id == key
 del key, val
 
-for n in ambiguous_generic_by_id.keys():
+for n in ambiguous_generic_by_id:
     assert ambiguous_rna_by_id[n].forward_table["GUU"] == "V"
     assert ambiguous_rna_by_id[n].forward_table["GUN"] == "V"
     if n != 23 :
