@@ -1,4 +1,5 @@
 # Copyright 2007-2010 by Peter Cock.  All rights reserved.
+# Revisions copyright 2010 by Uri Laserson.  All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
@@ -26,6 +27,7 @@
 
 import warnings
 import os
+import re
 from Bio.Seq import Seq
 from Bio.SeqRecord import SeqRecord
 from Bio.Alphabet import generic_alphabet, generic_protein
@@ -154,8 +156,11 @@ class InsdcScanner:
                 line = self.handle.readline()
                 break
             if line[2:self.FEATURE_QUALIFIER_INDENT].strip() == "":
-                raise ValueError("Expected a feature qualifier in line '%s'" % line)
-
+                #This is an empty feature line between qualifiers. Empty
+                #feature lines within qualifiers are handled below (ignored).
+                line = self.handle.readline()
+                continue
+            
             if skip:
                 line = self.handle.readline()
                 while line[:self.FEATURE_QUALIFIER_INDENT] == self.FEATURE_QUALIFIER_SPACER:
@@ -169,7 +174,6 @@ class InsdcScanner:
                     #over indenting the location and qualifiers.
                     feature_key, line = line[2:].strip().split(None,1)
                     feature_lines = [line]
-                    import warnings
                     warnings.warn("Overindented %s feature?" % feature_key)
                 else:
                     feature_key = line[2:self.FEATURE_QUALIFIER_INDENT].strip()
@@ -246,7 +250,8 @@ class InsdcScanner:
             feature_location = line.strip()
             while feature_location[-1:]==",":
                 #Multiline location, still more to come!
-                feature_location += iterator.next().strip()
+                line = iterator.next()
+                feature_location += line.strip()
 
             qualifiers=[]
 
@@ -515,6 +520,7 @@ class InsdcScanner:
 
                     yield record
 
+
 class EmblScanner(InsdcScanner):
     """For extracting chunks of information in EMBL files"""
 
@@ -555,10 +561,11 @@ class EmblScanner(InsdcScanner):
                 break
             assert self.line[:self.HEADER_WIDTH] == " " * self.HEADER_WIDTH, \
                    repr(self.line)
-            seq_lines.append("".join(line.split()[:-1]))
+            #Remove tailing number now, remove spaces later
+            seq_lines.append(line.rsplit(None,1)[0])
             line = self.handle.readline()
         self.line = line
-        return (misc_lines, "".join(seq_lines))
+        return (misc_lines, "".join(seq_lines).replace(" ", ""))
 
     def _feed_first_line(self, consumer, line):
         assert line[:self.HEADER_WIDTH].rstrip() == "ID"
@@ -638,7 +645,7 @@ class EmblScanner(InsdcScanner):
     def _feed_seq_length(self, consumer, text):
         length_parts = text.split()
         assert len(length_parts) == 2
-        assert length_parts[1].upper() in ["BP", "BP."]
+        assert length_parts[1].upper() in ["BP", "BP.", "AA."]
         consumer.size(length_parts[0])
 
     def _feed_header_lines(self, consumer, lines):
@@ -664,93 +671,82 @@ class EmblScanner(InsdcScanner):
         }
         #We have to handle the following specially:
         #RX (depending on reference type...)
-        lines = filter(None,lines)
-        line_iter = iter(lines)
-        try:
-            while True:
-                try:
-                    line = line_iter.next()
-                except StopIteration:
-                    break
-                if not line : break
-                line_type = line[:EMBL_INDENT].strip()
-                data = line[EMBL_INDENT:].strip()
-
-                if line_type == 'XX':
-                    pass
-                elif line_type == 'RN':
-                    # Reformat reference numbers for the GenBank based consumer
-                    # e.g. '[1]' becomes '1'
-                    if data[0] == "[" and data[-1] == "]" : data = data[1:-1]
-                    consumer.reference_num(data)
-                elif line_type == 'RP':
-                    # Reformat reference numbers for the GenBank based consumer
-                    # e.g. '1-4639675' becomes '(bases 1 to 4639675)'
-                    # and '160-550, 904-1055' becomes '(bases 160 to 550; 904 to 1055)'
-                    parts = [bases.replace("-"," to ").strip() for bases in data.split(",")]
-                    consumer.reference_bases("(bases %s)" % "; ".join(parts))
-                elif line_type == 'RT':
-                    #Remove the enclosing quotes and trailing semi colon.
-                    #Note the title can be split over multiple lines.
-                    if data.startswith('"'):
-                        data = data[1:]
-                    if data.endswith('";'):
-                        data = data[:-2]
-                    consumer.title(data)
-                elif line_type == 'RX':
-                    # EMBL support three reference types at the moment:
-                    # - PUBMED    PUBMED bibliographic database (NLM)
-                    # - DOI       Digital Object Identifier (International DOI Foundation)
-                    # - AGRICOLA  US National Agriculture Library (NAL) of the US Department
-                    #             of Agriculture (USDA)
-                    #
-                    # Format:
-                    # RX  resource_identifier; identifier.
-                    #
-                    # e.g.
-                    # RX   DOI; 10.1016/0024-3205(83)90010-3.
-                    # RX   PUBMED; 264242.
-                    #
-                    # Currently our reference object only supports PUBMED and MEDLINE
-                    # (as these were in GenBank files?).
-                    key, value = data.split(";",1)
-                    if value.endswith(".") : value = value[:-1]
-                    value = value.strip()
-                    if key == "PUBMED":
-                        consumer.pubmed_id(value)
-                    #TODO - Handle other reference types (here and in BioSQL bindings)
-                elif line_type == 'CC':
-                    # Have to pass a list of strings for this one (not just a string)
-                    consumer.comment([data])
-                elif line_type == 'DR':
-                    # Database Cross-reference, format:
-                    # DR   database_identifier; primary_identifier; secondary_identifier.
-                    #
-                    # e.g.
-                    # DR   MGI; 98599; Tcrb-V4.
-                    #
-                    # TODO - How should we store any secondary identifier?
-                    parts = data.rstrip(".").split(";")
-                    #Turn it into "database_identifier:primary_identifier" to
-                    #mimic the GenBank parser. e.g. "MGI:98599"
-                    consumer.dblink("%s:%s" % (parts[0].strip(),
-                                               parts[1].strip()))
-                elif line_type == 'RA':
-                    # Remove trailing ; at end of authors list
-                    consumer.authors(data.rstrip(";"))
-                elif line_type == 'PR':
-                    # Remove trailing ; at end of the project reference
-                    # In GenBank files this corresponds to the old PROJECT
-                    # line which is being replaced with the DBLINK line.
-                    consumer.project(data.rstrip(";"))
-                elif line_type in consumer_dict:
-                    #Its a semi-automatic entry!
-                    getattr(consumer, consumer_dict[line_type])(data)
-                else:
-                    if self.debug:
-                        print "Ignoring EMBL header line:\n%s" % line
-        except StopIteration:
-            raise ValueError("Problem with header")
+        for line in lines:
+            line_type = line[:EMBL_INDENT].strip()
+            data = line[EMBL_INDENT:].strip()
+            if line_type == 'XX':
+                pass
+            elif line_type == 'RN':
+                # Reformat reference numbers for the GenBank based consumer
+                # e.g. '[1]' becomes '1'
+                if data[0] == "[" and data[-1] == "]" : data = data[1:-1]
+                consumer.reference_num(data)
+            elif line_type == 'RP':
+                # Reformat reference numbers for the GenBank based consumer
+                # e.g. '1-4639675' becomes '(bases 1 to 4639675)'
+                # and '160-550, 904-1055' becomes '(bases 160 to 550; 904 to 1055)'
+                parts = [bases.replace("-"," to ").strip() for bases in data.split(",")]
+                consumer.reference_bases("(bases %s)" % "; ".join(parts))
+            elif line_type == 'RT':
+                #Remove the enclosing quotes and trailing semi colon.
+                #Note the title can be split over multiple lines.
+                if data.startswith('"'):
+                    data = data[1:]
+                if data.endswith('";'):
+                    data = data[:-2]
+                consumer.title(data)
+            elif line_type == 'RX':
+                # EMBL support three reference types at the moment:
+                # - PUBMED    PUBMED bibliographic database (NLM)
+                # - DOI       Digital Object Identifier (International DOI Foundation)
+                # - AGRICOLA  US National Agriculture Library (NAL) of the US Department
+                #             of Agriculture (USDA)
+                #
+                # Format:
+                # RX  resource_identifier; identifier.
+                #
+                # e.g.
+                # RX   DOI; 10.1016/0024-3205(83)90010-3.
+                # RX   PUBMED; 264242.
+                #
+                # Currently our reference object only supports PUBMED and MEDLINE
+                # (as these were in GenBank files?).
+                key, value = data.split(";",1)
+                if value.endswith(".") : value = value[:-1]
+                value = value.strip()
+                if key == "PUBMED":
+                    consumer.pubmed_id(value)
+                #TODO - Handle other reference types (here and in BioSQL bindings)
+            elif line_type == 'CC':
+                # Have to pass a list of strings for this one (not just a string)
+                consumer.comment([data])
+            elif line_type == 'DR':
+                # Database Cross-reference, format:
+                # DR   database_identifier; primary_identifier; secondary_identifier.
+                #
+                # e.g.
+                # DR   MGI; 98599; Tcrb-V4.
+                #
+                # TODO - How should we store any secondary identifier?
+                parts = data.rstrip(".").split(";")
+                #Turn it into "database_identifier:primary_identifier" to
+                #mimic the GenBank parser. e.g. "MGI:98599"
+                consumer.dblink("%s:%s" % (parts[0].strip(),
+                                           parts[1].strip()))
+            elif line_type == 'RA':
+                # Remove trailing ; at end of authors list
+                consumer.authors(data.rstrip(";"))
+            elif line_type == 'PR':
+                # Remove trailing ; at end of the project reference
+                # In GenBank files this corresponds to the old PROJECT
+                # line which is being replaced with the DBLINK line.
+                consumer.project(data.rstrip(";"))
+            elif line_type in consumer_dict:
+                #Its a semi-automatic entry!
+                getattr(consumer, consumer_dict[line_type])(data)
+            else:
+                if self.debug:
+                    print "Ignoring EMBL header line:\n%s" % line
         
     def _feed_misc_lines(self, consumer, lines):
         #TODO - Should we do something with the information on the SQ line(s)?
@@ -775,6 +771,101 @@ class EmblScanner(InsdcScanner):
         except StopIteration:
             raise ValueError("Problem in misc lines before sequence")
 
+
+class _ImgtScanner(EmblScanner):
+    """For extracting chunks of information in IMGT (EMBL like) files (PRIVATE).
+    
+    IMGT files are like EMBL files but in order to allow longer feature types
+    the features should be indented by 25 characters not 21 characters. In
+    practice the IMGT flat files tend to use either 21 or 25 characters, so we
+    must cope with both.
+    
+    This is private to encourage use of Bio.SeqIO rather than Bio.GenBank.
+    """
+
+    FEATURE_START_MARKERS = ["FH   Key             Location/Qualifiers",
+                             "FH   Key             Location/Qualifiers (from EMBL)",
+                             "FH   Key                 Location/Qualifiers",
+                             "FH"]
+
+    def parse_features(self, skip=False):
+        """Return list of tuples for the features (if present)
+
+        Each feature is returned as a tuple (key, location, qualifiers)
+        where key and location are strings (e.g. "CDS" and
+        "complement(join(490883..490885,1..879))") while qualifiers
+        is a list of two string tuples (feature qualifier keys and values).
+
+        Assumes you have already read to the start of the features table.
+        """
+        if self.line.rstrip() not in self.FEATURE_START_MARKERS:
+            if self.debug : print "Didn't find any feature table"
+            return []
+        
+        while self.line.rstrip() in self.FEATURE_START_MARKERS:
+            self.line = self.handle.readline()
+
+        bad_position_re = re.compile(r'([0-9]+)>{1}')
+        
+        features = []
+        line = self.line
+        while True:
+            if not line:
+                raise ValueError("Premature end of line during features table")
+            if line[:self.HEADER_WIDTH].rstrip() in self.SEQUENCE_HEADERS:
+                if self.debug : print "Found start of sequence"
+                break
+            line = line.rstrip()
+            if line == "//":
+                raise ValueError("Premature end of features table, marker '//' found")
+            if line in self.FEATURE_END_MARKERS:
+                if self.debug : print "Found end of features"
+                line = self.handle.readline()
+                break
+            if line[2:self.FEATURE_QUALIFIER_INDENT].strip() == "":
+                #This is an empty feature line between qualifiers. Empty
+                #feature lines within qualifiers are handled below (ignored).
+                line = self.handle.readline()
+                continue
+
+            if skip:
+                line = self.handle.readline()
+                while line[:self.FEATURE_QUALIFIER_INDENT] == self.FEATURE_QUALIFIER_SPACER:
+                    line = self.handle.readline()
+            else:
+                assert line[:2] == "FT"
+                try:
+                    feature_key, location_start = line[2:].strip().split()
+                except ValueError:
+                    #e.g. "FT   TRANSMEMBRANE-REGION2163..2240\n"
+                    #Assume indent of 25 as per IMGT spec, with the location
+                    #start in column 26 (one-based).
+                    feature_key = line[2:25].strip()
+                    location_start = line[25:].strip()
+                feature_lines = [location_start]
+                line = self.handle.readline()
+                while line[:self.FEATURE_QUALIFIER_INDENT] == self.FEATURE_QUALIFIER_SPACER \
+                or line.rstrip() == "" : # cope with blank lines in the midst of a feature
+                    #Use strip to remove any harmless trailing white space AND and leading
+                    #white space (copes with 21 or 26 indents and orther variants)
+                    assert line[:2] == "FT"
+                    feature_lines.append(line[self.FEATURE_QUALIFIER_INDENT:].strip())
+                    line = self.handle.readline()
+                feature_key, location, qualifiers = \
+                                self.parse_feature(feature_key, feature_lines)
+                #Try to handle known problems with IMGT locations here:
+                if ">" in location:
+                    #Nasty hack for common IMGT bug, should be >123 not 123>
+                    #in a location string. At least here the meaning is clear, 
+                    #and since it is so common I don't want to issue a warning
+                    #warnings.warn("Feature location %s is invalid, "
+                    #              "moving greater than sign before position"
+                    #              % location)
+                    location = bad_position_re.sub(r'>\1',location)
+                features.append((feature_key, location, qualifiers))
+        self.line = line
+        return features
+
 class GenBankScanner(InsdcScanner):
     """For extracting chunks of information in GenBank files"""
 
@@ -823,12 +914,12 @@ class GenBankScanner(InsdcScanner):
                 break
             if len(line) > 9 and  line[9:10]!=' ':
                 raise ValueError("Sequence line mal-formed, '%s'" % line)
-            seq_lines.append(line[10:].replace(" ",""))
+            seq_lines.append(line[10:]) #remove spaces later
             line = self.handle.readline()
 
         self.line = line
         #Seq("".join(seq_lines), self.alphabet)
-        return (misc_lines,"".join(seq_lines))
+        return (misc_lines,"".join(seq_lines).replace(" ",""))
 
     def _feed_first_line(self, consumer, line):
         #####################################
