@@ -92,6 +92,8 @@ class MEMEMotif (Motif):
         inst._strand(strand)
         if self.length:
             inst._length(self.length)
+        else:
+            inst._length(len(sequence))
         if self.name:
             inst._motifname(self.name)
         self.add_instance(inst)
@@ -356,15 +358,16 @@ def __skip_unused_lines(handle):
         raise ValueError("Line does not start with '***':\n%s" % line)
 
 
-# Everything below is obsolete.
+# Everything below is deprecated.
 
 
 from Bio import File
 from Bio.ParserSupport import *
+import Bio
 
 
 class MEMEParser (AbstractParser):
-    """A parser for the text output of the MEME program (OBSOLETE).
+    """A parser for the text output of the MEME program (DEPRECATED).
     Parses the output into an object of the MEMERecord class.
     
     Methods:
@@ -379,11 +382,13 @@ class MEMEParser (AbstractParser):
     ...    for instance in motif.instances:
     ...        print instance.motif_name, instance.sequence_name, instance.strand, instance.pvalue
     
-    This class is OBSOLETE; please use the read() function in this module
+    This class is DEPRECATED; please use the read() function in this module
     instead.
     """
     def __init__ (self):
         """__init__ (self)"""
+        import warnings
+        warnings.warn("Bio.Motif.Parsers.MEME.MEMEParser is deprecated; please use the read() function in this module instead", Bio.BiopythonDeprecationWarning)
         self._scanner = _MEMEScanner()
         self._consumer = _MEMEConsumer()
     
@@ -395,15 +400,19 @@ class MEMEParser (AbstractParser):
 
 
 class _MEMEScanner:
-    """Scanner for MEME output (OBSOLETE).
+    """Scanner for MEME output (DEPRECATED).
     
     Methods:
     feed
     
-    This class is OBSOLETE; please use the read() function in this module
+    This class is DEPRECATED; please use the read() function in this module
     instead.
     """
     
+    def __init__(self):
+        import warnings
+        warnings.warn("Bio.Motif.Parsers.MEME._MEMEScanner is deprecated; please use the read() function in this module instead", Bio.BiopythonDeprecationWarning)
+
     def feed (self, handle, consumer):
         """
         Feeds in MEME output for scanning. handle should
@@ -465,16 +474,18 @@ class _MEMEScanner:
 
 class _MEMEConsumer:
     """
-    Consumer that can receive events from MEME Scanner (OBSOLETE).
+    Consumer that can receive events from MEME Scanner (DEPRECATED).
     
     This is the Consumer object that should be passed to the 
     MEME Scanner.
     
-    This class is OBSOLETE; please use the read() function in this module
+    This class is DEPRECATED; please use the read() function in this module
     instead.
     """
     
     def __init__ (self):
+        import warnings
+        warnings.warn("Bio.Motif.Parsers.MEME._MEMEConsumer is deprecated; please use the read() function in this module instead", Bio.BiopythonDeprecationWarning)
         self.current_motif = None
         self.sequence_names = []
         self.data = MEMERecord()
@@ -550,7 +561,7 @@ class _MEMEConsumer:
 
 class _MASTConsumer:
     """
-    Consumer that can receive events from _MASTScanner (OBSOLETE).
+    Consumer that can receive events from _MASTScanner (DEPRECATED).
     
     A _MASTConsumer parses lines from a mast text output file.
     The motif match diagrams are parsed using line buffering. 
@@ -559,10 +570,12 @@ class _MASTConsumer:
     If this variable isn't there, the TaggingConsumer barfs. In
     the _MASTScanner, None is passed in the place of this variable.
     
-    This class is OBSOLETE; please use the read() function in the module
+    This class is DEPRECATED; please use the read() function in the module
     Bio.Motif.Parsers.MAST instead.
     """
     def __init__ (self):
+        import warnings
+        warnings.warn("Bio.Motif.Parsers.MEME._MASTConsumer is deprecated; please use the read() function in the module Bio.Motif.Parsers.MAST instead", Bio.BiopythonDeprecationWarning)
         self.data = MASTRecord()
         self._current_seq = ""
         self._line_buffer = []
@@ -785,7 +798,7 @@ class _MASTConsumer:
 
 class MASTParser(AbstractParser):
     """
-    Parser for MAST text output (OBSOLETE).
+    Parser for MAST text output (DEPRECATED).
     HTML output cannot be parsed, yet. Returns a MASTRecord
     
     A MASTParser takes a file handle for a MAST text output file and 
@@ -803,13 +816,15 @@ class MASTParser(AbstractParser):
     >>>parser = MASTParser()
     >>>mast_record = parser.parse(f)
     >>>for motif in mast_record.motifs:
-    >>>    for instance in motif.instances:
-    >>>        print instance.motif_name, instance.sequence_name, instance.strand, instance.pvalue
+    ...    for instance in motif.instances:
+    ...        print instance.motif_name, instance.sequence_name, instance.strand, instance.pvalue
     
-    This class is OBSOLETE; please use the read() function in the module
+    This class is DEPRECATED; please use the read() function in the module
     Bio.Motif.Parsers.MAST instead.
     """
     def __init__ (self):
+        import warnings
+        warnings.warn("Bio.Motif.Parsers.MEME.MASTParser is deprecated; please use the read() function in the module Bio.Motif.Parsers.MAST instead", Bio.BiopythonDeprecationWarning)
         self._consumer = _MASTConsumer()
         self._scanner = _MASTScanner()
     
@@ -821,11 +836,15 @@ class MASTParser(AbstractParser):
 
 class _MASTScanner:
     """
-    Scanner for MAST text output (OBSOLETE).
+    Scanner for MAST text output (DEPRECATED).
     
-    This class is OBSOLETE; please use the read() function in the module
+    This class is DEPRECATED; please use the read() function in the module
     Bio.Motif.Parsers.MAST instead.
     """
+    def __init__(self):
+        import warnings
+        warnings.warn("Bio.Motif.Parsers.MEME._MASTScanner is deprecated; please use the read() function in the module Bio.Motif.Parsers.MAST instead", Bio.BiopythonDeprecationWarning)
+
     def feed (self, handle, consumer):
         if isinstance(handle, File.UndoHandle):
             uhandle = handle
@@ -900,7 +919,7 @@ class _MASTScanner:
 
 
 class MASTRecord:
-    """The class for holding the results from a MAST run (OBSOLETE).
+    """The class for holding the results from a MAST run (DEPRECATED).
     
     A MASTRecord holds data about matches between motifs and sequences.
     The motifs held by the MASTRecord are objects of the class MEMEMotif.
@@ -915,10 +934,12 @@ class MASTRecord:
     get_motif_by_name (motif_name): returns a MEMEMotif with the given
     name.
     
-    This class is OBSOLETE; please use the read() function in the module
+    This class is DEPRECATED; please use the read() function in the module
     Bio.Motif.Parsers.MAST instead.
     """
     def __init__ (self):
+        import warnings
+        warnings.warn("Bio.Motif.Parsers.MEME.MASTRecord is deprecated; please use the read() function in the module Bio.Motif.Parsers.MAST instead", Bio.BiopythonDeprecationWarning)
         self.sequences = []
         self.version = ""
         self.matches = []
