@@ -25,16 +25,15 @@ calculate      Calculate the probabilities of each class, given an observation.
 classify       Classify an observation into a class.
 
 """
-# To Do:
-# add code to help discretize data
-# use objects
 
-try:
-    from Numeric import *
-except ImportError, x:
-    raise ImportError, "This module requires Numeric (precursor to NumPy)"
+import numpy
 
-from Bio import mathfns, listfns
+def _contents(items):
+    term = 1.0/len(items)
+    counts = {}
+    for item in items:
+        counts[item] = counts.get(item,0) + term
+    return counts
 
 class NaiveBayes:
     """Holds information for a NaiveBayes classifier.
@@ -69,32 +68,29 @@ def calculate(nb, observation, scale=0):
 
     # Make sure the observation has the right dimensionality.
     if len(observation) != nb.dimensionality:
-        raise ValueError, "observation in %d dimension, but classifier in %d" \
-              % (len(observation), nb.dimensionality)
+        raise ValueError("observation in %d dimension, but classifier in %d" \
+                         % (len(observation), nb.dimensionality))
 
     # Calculate log P(observation|class) for every class.
-    lp_observation_class = []     # list of log P(observation|class)
-    for i in range(len(nb.classes)):
+    n  = len(nb.classes)
+    lp_observation_class = numpy.zeros(n)   # array of log P(observation|class)
+    for i in range(n):
         # log P(observation|class) = SUM_i log P(observation_i|class)
         probs = [None] * len(observation)
         for j in range(len(observation)):
             probs[j] = nb.p_conditional[i][j].get(observation[j], 0)
-        lprobs = [mathfns.safe_log(x, -10000) for x in probs]
-        lprob = sum(lprobs)
-        lp_observation_class.append(lprob)
+        lprobs = numpy.log(numpy.clip(probs, 1.e-300, 1.e+300))
+        lp_observation_class[i] = sum(lprobs)
 
     # Calculate log P(class).
-    lp_prior = map(math.log, nb.p_prior)
+    lp_prior = numpy.log(nb.p_prior)
 
     # Calculate log P(observation).
     lp_observation = 0.0          # P(observation)
     if scale:   # Only calculate this if requested.
         # log P(observation) = log SUM_i P(observation|class_i)P(class_i)
-        obs = zeros(len(nb.classes), Float32)
-        for i in range(len(nb.classes)):
-            obs[i] = mathfns.safe_exp(lp_prior[i]+lp_observation_class[i],
-                                      under=1E-300)
-        lp_observation = math.log(sum(obs))
+        obs = numpy.exp(numpy.clip(lp_prior+lp_observation_class,-700,+700))
+        lp_observation = numpy.log(sum(obs))
 
     # Calculate log P(class|observation).
     lp_class_observation = {}      # Dict of class : log P(class|observation)
@@ -130,9 +126,9 @@ def train(training_set, results, priors=None, typecode=None):
 
     """
     if not len(training_set):
-        raise ValueError, "No data in the training set."
+        raise ValueError("No data in the training set.")
     if len(training_set) != len(results):
-        raise ValueError, "training_set and results should be parallel lists."
+        raise ValueError("training_set and results should be parallel lists.")
 
     # If no typecode is specified, try to pick a reasonable one.  If
     # training_set is a Numeric array, then use that typecode.
@@ -143,21 +139,23 @@ def train(training_set, results, priors=None, typecode=None):
     # dimensionality.
     dimensions = [len(x) for x in training_set]
     if min(dimensions) != max(dimensions):
-        raise ValueError, "observations have different dimensionality"
+        raise ValueError("observations have different dimensionality")
 
     nb = NaiveBayes()
     nb.dimensionality = dimensions[0]
     
-    # Get a list of all the classes.
-    nb.classes = listfns.items(results)
-    nb.classes.sort()   # keep it tidy
-    
-    # Estimate the prior probabilities for the classes.
+    # Get a list of all the classes, and
+    # estimate the prior probabilities for the classes.
     if priors is not None:
         percs = priors
+        nb.classes = list(set(results))
     else:
-        percs = listfns.contents(results)
-    nb.p_prior = zeros(len(nb.classes), Float16)
+        class_freq = _contents(results)
+        nb.classes = class_freq.keys()
+        percs = class_freq
+    nb.classes.sort()   # keep it tidy
+
+    nb.p_prior = numpy.zeros(len(nb.classes))
     for i in range(len(nb.classes)):
         nb.p_prior[i] = percs[nb.classes[i]]
 
@@ -167,7 +165,9 @@ def train(training_set, results, priors=None, typecode=None):
     # were guaranteed to be a matrix.  However, this may not be the
     # case, because the client may be hacking up a sparse matrix or
     # something.
-    c2i = listfns.itemindex(nb.classes)      # class to index of class
+    c2i = {}      # class to index of class
+    for index, key in enumerate(nb.classes):
+        c2i[key] = index
     observations = [[] for c in nb.classes]  # separate observations by class
     for i in range(len(results)):
         klass, obs = results[i], training_set[i]
@@ -175,7 +175,7 @@ def train(training_set, results, priors=None, typecode=None):
     # Now make the observations Numeric matrics.
     for i in range(len(observations)):
         # XXX typecode must be specified!
-        observations[i] = asarray(observations[i], typecode)
+        observations[i] = numpy.asarray(observations[i], typecode)
 
     # Calculate P(value|class,dim) for every class.
     # This is a good loop to optimize.
@@ -191,5 +191,40 @@ def train(training_set, results, priors=None, typecode=None):
             #values = list(values) + range(len(nb.classes))  # XXX add 1
             
             # Estimate P(value|class,dim)
-            nb.p_conditional[i][j] = listfns.contents(values)
+            nb.p_conditional[i][j] = _contents(values)
     return nb
+
+if __name__ == "__main__":
+    # Car data from example 'Naive Bayes Classifier example' by Eric Meisner November 22, 2003
+    # http://www.inf.u-szeged.hu/~ormandi/teaching/mi2/02-naiveBayes-example.pdf
+    xcar=[
+        ['Red',    'Sports', 'Domestic'],
+        ['Red',    'Sports', 'Domestic'],
+        ['Red',    'Sports', 'Domestic'],
+        ['Yellow', 'Sports', 'Domestic'],
+        ['Yellow', 'Sports', 'Imported'],
+        ['Yellow', 'SUV',    'Imported'],
+        ['Yellow', 'SUV',    'Imported'],
+        ['Yellow', 'SUV',    'Domestic'],
+        ['Red',    'SUV',    'Imported'],
+        ['Red',    'Sports', 'Imported']
+    ]
+
+    ycar=[
+        'Yes',
+        'No',
+        'Yes',
+        'No',
+        'Yes',
+        'No',
+        'Yes',
+        'No',
+        'No',
+        'Yes'
+    ]
+
+    carmodel = train(xcar, ycar)
+    carresult = classify(carmodel, ['Red', 'Sports', 'Domestic'])
+    print 'Is Yes?', carresult
+    carresult = classify(carmodel, ['Red', 'SUV', 'Domestic'])
+    print 'Is No?', carresult
