@@ -3,20 +3,10 @@
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
 
-from __future__ import generators
+"""Use the DSSP program to calculate secondary structure and accessibility.
 
-import os
-import tempfile
-from Bio.PDB import *
-from PDBExceptions import PDBException
-from AbstractPropertyMap import AbstractResiduePropertyMap
-import re
-
-
-__doc__="""
-Use the DSSP program to calculate secondary structure and accessibility.
-You need to have a working version of DSSP (and a license, free for 
-academic use) in order to use this. For DSSP, see U{http://www.cmbi.kun.nl/gv/dssp/}.
+You need to have a working version of DSSP (and a license, free for academic
+use) in order to use this. For DSSP, see U{http://www.cmbi.kun.nl/gv/dssp/}.
 
 The DSSP codes for secondary structure used here are:
 
@@ -30,6 +20,17 @@ The DSSP codes for secondary structure used here are:
     - -        None
 """
 
+import os
+import re
+import tempfile
+
+from Bio.SCOP.Raf import to_one_letter_code
+
+from Bio.PDB.AbstractPropertyMap import AbstractResiduePropertyMap
+from Bio.PDB.PDBExceptions import PDBException
+from Bio.PDB.PDBParser import PDBParser
+
+
 # Match C in DSSP
 _dssp_cys=re.compile('[a-z]')
 
@@ -193,7 +194,7 @@ class DSSP(AbstractResiduePropertyMap):
                 if _dssp_cys.match(aa):
                     aa='C'
             if not (resname==aa):
-                raise PDBException, "Structure/DSSP mismatch at "+str(res) 
+                raise PDBException("Structure/DSSP mismatch at "+str(res))
             dssp_map[key]=((res, ss, acc, rel_acc))
             dssp_list.append((res, ss, acc, rel_acc))
         AbstractResiduePropertyMap.__init__(self, dssp_map, dssp_keys, dssp_list)
@@ -217,7 +218,7 @@ if __name__=="__main__":
 
     print len(d)
 
-    print d.has_key(('A', 1))
+    print ('A', 1) in d
 
     print d[('A', 1)]
 
