@@ -3,18 +3,12 @@
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
 
-import os
-import numpy
+"""Classify protein backbone structure according to Kolodny et al's fragment
+libraries.
 
-from Bio.SVDSuperimposer import SVDSuperimposer
-from Bio.PDB import *
-from Bio.PDB.PDBExceptions import PDBException
-
-__doc__="""
-Classify protein backbone structure according to Kolodny et al's fragment
-libraries. It can be regarded as a form of objective secondary structure
-classification. Only fragments of length 5 or 7 are supported (ie. there is a
-'central' residue).
+It can be regarded as a form of objective secondary structure classification.
+Only fragments of length 5 or 7 are supported (ie. there is a 'central'
+residue).
 
 Full reference:
 
@@ -31,12 +25,20 @@ You need these files to use this module.
 The following example uses the library with 10 fragments of length 5.
 The library files can be found in directory 'fragment_data'.
 
-    >>> model=structure[0]
-    >>> fm=FragmentMapper(lsize=10, flength=5, dir="fragment_data")
-    >>> fm.map(model)
-    >>> fragment=fm[residue]
+    >>> model = structure[0]
+    >>> fm = FragmentMapper(model, lsize=10, flength=5, dir="fragment_data")
+    >>> fragment = fm[residue]
 """
 
+import numpy
+
+from Bio.SVDSuperimposer import SVDSuperimposer
+
+from Bio.PDB import Selection
+from Bio.PDB.PDBExceptions import PDBException
+from Bio.PDB.PDBParser import PDBParser
+from Bio.PDB.Polypeptide import PPBuilder
+
 
 # fragment file (lib_SIZE_z_LENGTH.txt)
 # SIZE=number of fragments
@@ -282,16 +284,29 @@ class FragmentMapper:
                         index=i-self.edge
                         assert(index>=0)
                         fd[res]=mflist[index]
-            except "CHAINBREAK":
-                # Funny polypeptide - skip
-                pass
+            except PDBException, why:
+                if why == 'CHAINBREAK':
+                    # Funny polypeptide - skip
+                    pass
+                else:
+                    raise PDBException(why)
         return fd
 
     def has_key(self, res):
-        """
+        """(Obsolete)
+
         @type res: L{Residue}
         """
-        return self.fd.has_key(res)
+        import warnings
+        warnings.warn("has_key is obsolete; use 'res in object' instead", PendingDeprecationWarning)
+        return (res in self)
+
+    def __contains__(self, res):
+        """True if the given residue is in any of the mapped fragments.
+
+        @type res: L{Residue}
+        """
+        return (res in self.fd)
 
     def __getitem__(self, res):
         """
@@ -317,7 +332,7 @@ if __name__=="__main__":
     for r in Selection.unfold_entities(m, "R"):
 
         print r,
-        if fm.has_key(r):
+        if r in fm:
             print fm[r]
         else:
             print
