@@ -3,7 +3,7 @@
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.  
 
-__doc__="Some Bio.PDB specific exceptions."
+"""Some Bio.PDB-specific exceptions."""
 
 # General error
 class PDBException(Exception):
