@@ -11,7 +11,7 @@ __docformat__ = "epytext en"
 
 import warnings
 
-import BaseTree
+from Bio.Phylo import BaseTree
 
 
 class Tree(BaseTree.Tree):
@@ -24,12 +24,11 @@ class Tree(BaseTree.Tree):
 
 
 class Clade(BaseTree.Clade):
-    """Newick Clade (subtree) object."""
+    """Newick Clade (sub-tree) object."""
 
     def __init__(self, branch_length=1.0, name=None, clades=None,
                  confidence=None, comment=None):
         BaseTree.Clade.__init__(self, branch_length=branch_length,
-                name=name, clades=clades)
-        self.confidence = confidence
+                name=name, clades=clades, confidence=confidence)
         self.comment = comment
 
