@@ -116,8 +116,9 @@ def _check_bases(seq_string):
 def check_bases(seq_string):
     """Check characters in a string (DEPRECATED)."""
     import warnings
+    import Bio
     warnings.warn("The check_bases function has been deprecated, and will be"
-                  "removed in a future release of Biopython.", DeprecationWarning)
+                  "removed in a future release of Biopython.", Bio.BiopythonDeprecationWarning)
     return _check_bases(seq_string)
 
 matching = {'A' : 'ARWMHVDN', 'C' : 'CYSMHBVN', 'G' : 'GRSKBVDN',
@@ -323,16 +324,28 @@ class RestrictionType(type):
         length of the recognition site."""
         return cls.size
     
+    def __hash__(cls):
+        #Python default is to use id(...)
+        #This is consistent with the __eq__ implementation
+        return id(cls)
+    
     def __eq__(cls, other):
         """RE == other -> bool
 
-        True if RE and other are the same enzyme."""
-        return other is cls
+        True if RE and other are the same enzyme.
+        
+        Specifically this checks they are the same Python object.
+        """
+        #assert (id(cls)==id(other)) == (other is cls) == (cls is other)
+        return id(cls)==id(other)
 
     def __ne__(cls, other):
         """RE != other -> bool.
         isoschizomer strict, same recognition site, same restriction -> False
-        all the other-> True"""
+        all the other-> True
+        
+        WARNING - This is not the inverse of the __eq__ method.
+        """
         if not isinstance(other, RestrictionType):
             return True
         elif cls.charac == other.charac:
@@ -1859,7 +1872,7 @@ class RestrictionBatch(set):
         func return True."""
         d = [x for x in itertools.ifilter(func, self)]
         new = RestrictionBatch()
-        new._data = dict(map(None, d, [True]*len(d)))
+        new._data = dict(zip(d, [True]*len(d)))
         return new
 
     def add_supplier(self, letter):
@@ -1964,7 +1977,7 @@ class RestrictionBatch(set):
             return True
         d = [k for k in itertools.ifilter(splittest, self)]
         new = RestrictionBatch()
-        new._data = dict(map(None, d, [True]*len(d)))
+        new._data = dict(zip(d, [True]*len(d)))
         return new
       
     def elements(self):
@@ -2411,6 +2424,11 @@ AllEnzymes = CommOnly | NonComm
 #   Now, place the enzymes in locals so they can be imported.
 #
 names = [str(x) for x in AllEnzymes]
-locals().update(dict(map(None, names, AllEnzymes)))
+try:
+    del x
+except NameError:
+    #Scoping changed in Python 3, the variable isn't leaked
+    pass
+locals().update(dict(zip(names, AllEnzymes)))
 __all__=['FormattedSeq', 'Analysis', 'RestrictionBatch','AllEnzymes','CommOnly','NonComm']+names
-del k, x, enzymes, TYPE, bases, names
+del k, enzymes, TYPE, bases, names
