@@ -1,6 +1,6 @@
 # Copyright 2000-2002 Brad Chapman.
 # Copyright 2004-2005 by M de Hoon.
-# Copyright 2007-2009 by Peter Cock.
+# Copyright 2007-2010 by Peter Cock.
 # All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
@@ -17,10 +17,10 @@ import string #for maketrans only
 import array
 import sys
 
-import Alphabet
-from Alphabet import IUPAC
+from Bio import Alphabet
+from Bio.Alphabet import IUPAC
 from Bio.SeqRecord import SeqRecord
-from Data.IUPACData import ambiguous_dna_complement, ambiguous_rna_complement
+from Bio.Data.IUPACData import ambiguous_dna_complement, ambiguous_rna_complement
 from Bio.Data import CodonTable
 
 def _maketrans(complement_mapping):
@@ -41,7 +41,10 @@ def _maketrans(complement_mapping):
     after  = ''.join(complement_mapping.values())
     before = before + before.lower()
     after  = after + after.lower()
-    return string.maketrans(before, after)
+    if sys.version_info[0] == 3 :
+        return str.maketrans(before, after)
+    else:
+        return string.maketrans(before, after)
 
 _dna_complement_table = _maketrans(ambiguous_dna_complement)
 _rna_complement_table = _maketrans(ambiguous_rna_complement)
@@ -56,6 +59,12 @@ class Seq(object):
     The Seq object provides a number of string like methods (such as count,
     find, split and strip), which are alphabet aware where appropriate.
 
+    In addition to the string like sequence, the Seq object has an alphabet
+    property. This is an instance of an Alphabet class from Bio.Alphabet,
+    for example generic DNA, or IUPAC DNA. This describes the type of molecule
+    (e.g. RNA, DNA, protein) and may also indicate the expected symbols
+    (letters).
+
     The Seq object also provides some biological methods, such as complement,
     reverse_complement, transcribe, back_transcribe and translate (which are
     not applicable to sequences with a protein alphabet).
@@ -81,10 +90,14 @@ class Seq(object):
         Seq('MKQHKAMIVALIVICITAVVAALVTRKDLCEVHIRTGQTEVAVF', IUPACProtein())
         >>> print my_seq
         MKQHKAMIVALIVICITAVVAALVTRKDLCEVHIRTGQTEVAVF
+        >>> my_seq.alphabet
+        IUPACProtein()
+
         """
         # Enforce string storage
-        assert (type(data) == type("") or # must use a string
-                type(data) == type(u""))  # but can be a unicode string
+        if not isinstance(data, basestring):
+            raise TypeError("The sequence data given to a Seq object should "
+                            "be a string (not another Seq object etc)")
         self._data = data
         self.alphabet = alphabet  # Seq API requirement
  
@@ -92,7 +105,7 @@ class Seq(object):
     # Note this is read only since the Seq object is meant to be imutable
     @property
     def data(self) :
-        """Sequence as a string (OBSOLETE/DEPRECATED).
+        """Sequence as a string (DEPRECATED).
 
         This is a read only property provided for backwards compatility with
         older versions of Biopython (as is the tostring() method). We now
@@ -107,13 +120,18 @@ class Seq(object):
         >>> from Bio.Seq import Seq
         >>> from Bio.Alphabet import generic_dna
         >>> my_seq = Seq("ACGT", generic_dna)
-        >>> str(my_seq) == my_seq.tostring() == my_seq.data == "ACGT"
+        >>> str(my_seq) == my_seq.tostring() == "ACGT"
         True
         >>> my_seq.data = "AAAA"
         Traceback (most recent call last):
            ...
         AttributeError: can't set attribute
         """
+        import warnings
+        import Bio
+        warnings.warn("Accessing the .data attribute is deprecated. Please "
+                      "use str(my_seq) or my_seq.tostring() instead of "
+                      "my_seq.data.", Bio.BiopythonDeprecationWarning)
         return str(self)
 
     def __repr__(self):
@@ -127,7 +145,7 @@ class Seq(object):
                                    repr(self.alphabet))
         else:
             return "%s(%s, %s)" % (self.__class__.__name__,
-                                  repr(self.data),
+                                  repr(self._data),
                                    repr(self.alphabet))
     def __str__(self):
         """Returns the full sequence as a python string, use str(my_seq).
@@ -139,6 +157,13 @@ class Seq(object):
         """
         return self._data
 
+    def __hash__(self):
+        """Hash for comparison.
+
+        See the __cmp__ documentation - we plan to change this!
+        """
+        return id(self) #Currently use object identity for equality testing
+    
     def __cmp__(self, other):
         """Compare the sequence to another sequence or a string (README).
 
@@ -285,10 +310,16 @@ class Seq(object):
             raise TypeError
 
     def tostring(self):                            # Seq API requirement
-        """Returns the full sequence as a python string (OBSOLETE).
+        """Returns the full sequence as a python string (semi-obsolete).
 
         Although not formally deprecated, you are now encouraged to use
         str(my_seq) instead of my_seq.tostring()."""
+        #TODO - Fix all places elsewhere in Biopython using this method,
+        #then start deprecation process?
+        #import warnings
+        #warnings.warn("This method is obsolete; please use str(my_seq) "
+        #              "instead of my_seq.tostring().",
+        #              PendingDeprecationWarning)
         return str(self)
     
     def tomutable(self):   # Needed?  Or use a function?
@@ -851,8 +882,9 @@ class Seq(object):
 
         Arguments:
          - table - Which codon table to use?  This can be either a name
-                   (string) or an NCBI identifier (integer).  This defaults
-                   to the "Standard" table.
+                   (string), an NCBI identifier (integer), or a CodonTable
+                   object (useful for non-standard genetic codes).  This
+                   defaults to the "Standard" table.
          - stop_symbol - Single character string, what to use for terminators.
                          This defaults to the asterisk, "*".
          - to_stop - Boolean, defaults to False meaning do a full translation
@@ -918,10 +950,6 @@ class Seq(object):
         NOTE - This does NOT behave like the python string's translate
         method.  For that use str(my_seq).translate(...) instead.
         """
-        try:
-            table_id = int(table)
-        except ValueError:
-            table_id = None
         if isinstance(table, str) and len(table)==256:
             raise ValueError("The Seq object translate method DOES NOT take " \
                              + "a 256 character string mapping table like " \
@@ -930,25 +958,39 @@ class Seq(object):
         if isinstance(Alphabet._get_base_alphabet(self.alphabet),
                       Alphabet.ProteinAlphabet):
             raise ValueError("Proteins cannot be translated!")
-        if self.alphabet==IUPAC.unambiguous_dna:
-            #Will use standard IUPAC protein alphabet, no need for X
-            if table_id is None:
+        try:
+            table_id = int(table)
+        except ValueError:
+            #Assume its a table name
+            if self.alphabet==IUPAC.unambiguous_dna:
+                #Will use standard IUPAC protein alphabet, no need for X
                 codon_table = CodonTable.unambiguous_dna_by_name[table]
-            else:
-                codon_table = CodonTable.unambiguous_dna_by_id[table_id]
-        elif self.alphabet==IUPAC.unambiguous_rna:
-            #Will use standard IUPAC protein alphabet, no need for X
-            if table_id is None:
+            elif self.alphabet==IUPAC.unambiguous_rna:
+                #Will use standard IUPAC protein alphabet, no need for X
                 codon_table = CodonTable.unambiguous_rna_by_name[table]
             else:
-                codon_table = CodonTable.unambiguous_rna_by_id[table_id]
-        else:
-            #This will use the extend IUPAC protein alphabet with X etc.
-            #The same table can be used for RNA or DNA (we use this for
-            #translating strings).
-            if table_id is None:
+                #This will use the extended IUPAC protein alphabet with X etc.
+                #The same table can be used for RNA or DNA (we use this for
+                #translating strings).
                 codon_table = CodonTable.ambiguous_generic_by_name[table]
+        except (AttributeError, TypeError):
+            #Assume its a CodonTable object
+            if isinstance(table, CodonTable.CodonTable):
+                codon_table = table
             else:
+                raise ValueError('Bad table argument')
+        else:
+            #Assume its a table ID
+            if self.alphabet==IUPAC.unambiguous_dna:
+                #Will use standard IUPAC protein alphabet, no need for X
+                codon_table = CodonTable.unambiguous_dna_by_id[table_id]
+            elif self.alphabet==IUPAC.unambiguous_rna:
+                #Will use standard IUPAC protein alphabet, no need for X
+                codon_table = CodonTable.unambiguous_rna_by_id[table_id]
+            else:
+                #This will use the extended IUPAC protein alphabet with X etc.
+                #The same table can be used for RNA or DNA (we use this for
+                #translating strings).
                 codon_table = CodonTable.ambiguous_generic_by_id[table_id]
         protein = _translate_str(str(self), codon_table, \
                                  stop_symbol, to_stop, cds)
@@ -1173,13 +1215,49 @@ class UnknownSeq(Seq):
         return other + Seq(str(self), self.alphabet)
 
     def __getitem__(self, index):
+        """Get a subsequence from the UnknownSeq object.
+        
+        >>> unk = UnknownSeq(8, character="N")
+        >>> print unk[:]
+        NNNNNNNN
+        >>> print unk[5:3]
+        <BLANKLINE>
+        >>> print unk[1:-1]
+        NNNNNN
+        >>> print unk[1:-1:2]
+        NNN
+        """
         if isinstance(index, int):
             #TODO - Check the bounds without wasting memory
             return str(self)[index]
+        old_length = self._length
+        step = index.step
+        if step is None or step == 1:
+            #This calculates the length you'd get from ("N"*old_length)[index]
+            start = index.start
+            end = index.stop
+            if start is None:
+                start = 0
+            elif start < 0:
+                start = max(0, old_length + start)
+            elif start > old_length:
+                start = old_length
+            if end is None:
+                end = old_length
+            elif end < 0:
+                end = max(0, old_length + end)
+            elif end > old_length:
+                end = old_length
+            new_length = max(0, end-start)
+        elif step == 0:
+            raise ValueError("slice step cannot be zero")
         else:
-            #TODO - Work out the length without wasting memory
-            return UnknownSeq(len(("#"*self._length)[index]),
-                              self.alphabet, self._character)
+            #TODO - handle step efficiently
+            new_length = len(("X"*old_length)[index])
+        #assert new_length == len(("X"*old_length)[index]), \
+        #       (index, start, end, step, old_length,
+        #        new_length, len(("X"*old_length)[index]))
+        return UnknownSeq(new_length, self.alphabet, self._character)
 
     def count(self, sub, start=0, end=sys.maxint):
         """Non-overlapping count method, like that of a python string.
@@ -1446,8 +1524,12 @@ class MutableSeq(object):
     or biological methods as the Seq object.
     """
     def __init__(self, data, alphabet = Alphabet.generic_alphabet):
-        if type(data) == type(""):
-            self.data = array.array("c", data)
+        if sys.version_info[0] == 3:
+            self.array_indicator = "u"
+        else:
+            self.array_indicator = "c"
+        if isinstance(data, str): #TODO - What about unicode?
+            self.data = array.array(self.array_indicator, data)
         else:
             self.data = data   # assumes the input is an array
         self.alphabet = alphabet
@@ -1552,7 +1634,8 @@ class MutableSeq(object):
             elif isinstance(value, type(self.data)):
                 self.data[index] = value
             else:
-                self.data[index] = array.array("c", str(value))
+                self.data[index] = array.array(self.array_indicator,
+                                               str(value))
 
     def __delitem__(self, index):
         #Note since Python 2.0, __delslice__ is deprecated
@@ -1721,7 +1804,7 @@ class MutableSeq(object):
         c = dict([(x.lower(), y.lower()) for x,y in d.iteritems()])
         d.update(c)
         self.data = map(lambda c: d[c], self.data)
-        self.data = array.array('c', self.data)
+        self.data = array.array(self.array_indicator, self.data)
         
     def reverse_complement(self):
         """Modify the mutable sequence to take on its reverse complement.
@@ -1745,7 +1828,7 @@ class MutableSeq(object):
                 self.data.append(c)
 
     def tostring(self):
-        """Returns the full sequence as a python string.
+        """Returns the full sequence as a python string (semi-obsolete).
 
         Although not formally deprecated, you are now encouraged to use
         str(my_seq) instead of my_seq.tostring().
@@ -1766,8 +1849,8 @@ class MutableSeq(object):
 
         >>> from Bio.Seq import Seq
         >>> from Bio.Alphabet import IUPAC
-        >>> my_mseq = MutableSeq("MKQHKAMIVALIVICITAVVAAL", \
-                                 IUPAC.protein)
+        >>> my_mseq = MutableSeq("MKQHKAMIVALIVICITAVVAAL", 
+        ...                      IUPAC.protein)
         >>> my_mseq
         MutableSeq('MKQHKAMIVALIVICITAVVAAL', IUPACProtein())
         >>> my_mseq.toseq()
@@ -1925,9 +2008,10 @@ def translate(sequence, table="Standard", stop_symbol="*", to_stop=False,
     MutableSeq, returns a Seq object with a protein alphabet.
 
     Arguments:
-     - table - Which codon table to use?  This can be either a name
-               (string) or an NCBI identifier (integer).  Defaults
-               to the "Standard" table.
+     - table - Which codon table to use?  This can be either a name (string),
+               an NCBI identifier (integer), or a CodonTable object (useful
+               for non-standard genetic codes).  Defaults to the "Standard"
+               table.
      - stop_symbol - Single character string, what to use for any
                      terminators, defaults to the asterisk, "*".
      - to_stop - Boolean, defaults to False meaning do a full
@@ -1996,6 +2080,11 @@ def translate(sequence, table="Standard", stop_symbol="*", to_stop=False,
             codon_table = CodonTable.ambiguous_generic_by_id[int(table)]
         except ValueError:
             codon_table = CodonTable.ambiguous_generic_by_name[table]
+        except (AttributeError, TypeError):
+            if isinstance(table, CodonTable.CodonTable):
+                codon_table = table
+            else:
+                raise ValueError('Bad table argument')
         return _translate_str(sequence, codon_table, stop_symbol, to_stop, cds)
       
 def reverse_complement(sequence):
@@ -2033,11 +2122,15 @@ def reverse_complement(sequence):
     return sequence.translate(ttable)[::-1]
 
 def _test():
-    """Run the Bio.Seq module's doctests."""
-    print "Runing doctests..."
-    import doctest
-    doctest.testmod()
-    print "Done"
+    """Run the Bio.Seq module's doctests (PRIVATE)."""
+    if sys.version_info[0:2] == (3,1):
+        print "Not running Bio.Seq doctest on Python 3.1"
+        print "See http://bugs.python.org/issue7490"
+    else:
+        print "Runing doctests..."
+        import doctest
+        doctest.testmod(optionflags=doctest.IGNORE_EXCEPTION_DETAIL)
+        print "Done"
 
 if __name__ == "__main__":
     _test()
