@@ -48,14 +48,33 @@ class _IndexedSeqFileDict(dict):
     Note that this dictionary is essentially read only. You cannot
     add or change values, pop values, nor clear the dictionary.
     """
-    def __init__(self, filename, alphabet, key_function, mode="rU"):
+    def __init__(self, filename, format, alphabet, key_function):
         #Use key_function=None for default value
         dict.__init__(self) #init as empty dict!
+        if format in SeqIO._BinaryFormats:
+            mode = "rb"
+        else:
+            mode = "rU"
         self._handle = open(filename, mode)
         self._alphabet = alphabet
-        self._format = ""
+        self._format = format
         self._key_function = key_function
-        #Now scan it in a subclassed method, and set the format!
+        if key_function:
+            offset_iter = ((key_function(k),o) for (k,o) in self._build())
+        else:
+            offset_iter = self._build()
+        for key, offset in offset_iter:
+            if key in self:
+                raise ValueError("Duplicate key '%s'" % key)
+            else:
+                dict.__setitem__(self, key, offset)
+    
+    def _build(self):
+        """Actually scan the file identifying records and offsets (PRIVATE).
+        
+        Returns an iterator giving tuples of record names and their offsets.
+        """
+        pass
 
     def __repr__(self):
         return "SeqIO.index('%s', '%s', alphabet=%s, key_function=%s)" \
@@ -68,69 +87,53 @@ class _IndexedSeqFileDict(dict):
         else:
             return "{}"
 
-    def _record_key(self, identifier, seek_position):
-        """Used by subclasses to record file offsets for identifiers (PRIVATE).
+    if hasattr(dict, "iteritems"):
+        #Python 2, use iteritems but not items etc
+        def values(self):
+            """Would be a list of the SeqRecord objects, but not implemented.
 
-        This will apply the key_function (if given) to map the record id
-        string to the desired key.
+            In general you can be indexing very very large files, with millions
+            of sequences. Loading all these into memory at once as SeqRecord
+            objects would (probably) use up all the RAM. Therefore we simply
+            don't support this dictionary method.
+            """
+            raise NotImplementedError("Due to memory concerns, when indexing a "
+                                      "sequence file you cannot access all the "
+                                      "records at once.")
 
-        This will raise a ValueError if a key (record id string) occurs
-        more than once.
-        """
-        if self._key_function:
-            key = self._key_function(identifier)
-        else:
-            key = identifier
-        if key in self:
-            raise ValueError("Duplicate key '%s'" % key)
-        else:
-            dict.__setitem__(self, key, seek_position)
+        def items(self):
+            """Would be a list of the (key, SeqRecord) tuples, but not implemented.
 
-    def values(self):
-        """Would be a list of the SeqRecord objects, but not implemented.
+            In general you can be indexing very very large files, with millions
+            of sequences. Loading all these into memory at once as SeqRecord
+            objects would (probably) use up all the RAM. Therefore we simply
+            don't support this dictionary method.
+            """
+            raise NotImplementedError("Due to memory concerns, when indexing a "
+                                      "sequence file you cannot access all the "
+                                      "records at once.")
 
-        In general you can be indexing very very large files, with millions
-        of sequences. Loading all these into memory at once as SeqRecord
-        objects would (probably) use up all the RAM. Therefore we simply
-        don't support this dictionary method.
-        """
-        raise NotImplementedError("Due to memory concerns, when indexing a "
-                                  "sequence file you cannot access all the "
-                                  "records at once.")
+        def iteritems(self):
+            """Iterate over the (key, SeqRecord) items."""
+            for key in self.__iter__():
+                yield key, self.__getitem__(key)
+    else:
+        #Python 3 - define items and values as iterators
+        def items(self):
+            """Iterate over the (key, SeqRecord) items."""
+            for key in self.__iter__():
+                yield key, self.__getitem__(key)
 
-    def items(self):
-        """Would be a list of the (key, SeqRecord) tuples, but not implemented.
+        def values(self):
+            """Iterate over the SeqRecord items."""
+            for key in self.__iter__():
+                yield self.__getitem__(key)
 
-        In general you can be indexing very very large files, with millions
-        of sequences. Loading all these into memory at once as SeqRecord
-        objects would (probably) use up all the RAM. Therefore we simply
-        don't support this dictionary method.
-        """
-        raise NotImplementedError("Due to memory concerns, when indexing a "
-                                  "sequence file you cannot access all the "
-                                  "records at once.")
-
-    def iteritems(self):
-        """Iterate over the (key, SeqRecord) items."""
-        for key in self.__iter__():
-            yield key, self.__getitem__(key)
 
     def __getitem__(self, key):
         """x.__getitem__(y) <==> x[y]"""
-        #For non-trivial file formats this must be over-ridden in the subclass
-        handle = self._handle
-        handle.seek(dict.__getitem__(self, key))
-        record = SeqIO.parse(handle, self._format, self._alphabet).next()
-        if self._key_function:
-            assert self._key_function(record.id) == key, \
-                   "Requested key %s, found record.id %s which has key %s" \
-                   % (repr(key), repr(record.id),
-                      repr(self._key_function(record.id)))
-        else:
-            assert record.id == key, \
-                   "Requested key %s, found record.id %s" \
-                   % (repr(key), repr(record.id))
-        return record
+        #Should be done by each sub-class
+        raise NotImplementedError("Not implemented for this file format (yet).")
 
     def get(self, k, d=None):
         """D.get(k[,d]) -> D[k] if k in D, else d.  d defaults to None."""
@@ -194,13 +197,12 @@ class _IndexedSeqFileDict(dict):
 
 class SffDict(_IndexedSeqFileDict) :
     """Indexed dictionary like access to a Standard Flowgram Format (SFF) file."""
-    def __init__(self, filename, alphabet, key_function) :
-        if alphabet is None:
-            alphabet = Alphabet.generic_dna
-        #On Unix, using mode="r" or "rb" works, "rU" does not.
-        #On Windows, only using mode="rb" works, "r" and "rU" fail.
-        _IndexedSeqFileDict.__init__(self, filename, alphabet, key_function, "rb")
+    def _build(self):
+        """Load any index block in the file, or build it the slow way (PRIVATE)."""
+        if self._alphabet is None:
+            self._alphabet = Alphabet.generic_dna
         handle = self._handle
+        #Record the what we'll need for parsing a record given its offset
         header_length, index_offset, index_length, number_of_reads, \
         self._flows_per_read, self._flow_chars, self._key_sequence \
             = SeqIO.SffIO._sff_file_header(handle)
@@ -208,7 +210,7 @@ class SffDict(_IndexedSeqFileDict) :
             #There is an index provided, try this the fast way:
             try :
                 for name, offset in SeqIO.SffIO._sff_read_roche_index(handle) :
-                    self._record_key(name, offset)
+                    yield name, offset
                 assert len(self) == number_of_reads, \
                        "Indexed %i records, expected %i" \
                        % (len(self), number_of_reads)
@@ -216,7 +218,7 @@ class SffDict(_IndexedSeqFileDict) :
             except ValueError, err :
                 import warnings
                 warnings.warn("Could not parse the SFF index: %s" % err)
-                dict.clear(self) #reset in case partially populated
+                assert len(self)==0, "Partially populated index"
                 handle.seek(0)
         else :
             #TODO - Remove this debug warning?
@@ -224,57 +226,97 @@ class SffDict(_IndexedSeqFileDict) :
             warnings.warn("No SFF index, doing it the slow way")
         #Fall back on the slow way!
         for name, offset in SeqIO.SffIO._sff_do_slow_index(handle) :
-            #print "%s -> %i" % (name, offset)
-            self._record_key(name, offset)
+            yield name, offset
         assert len(self) == number_of_reads, \
                "Indexed %i records, expected %i" % (len(self), number_of_reads)
 
     def __getitem__(self, key) :
         handle = self._handle
         handle.seek(dict.__getitem__(self, key))
-        record = SeqIO.SffIO._sff_read_seq_record(handle,
-                                                  self._flows_per_read,
-                                                  self._flow_chars,
-                                                  self._key_sequence,
-                                                  self._alphabet)
-        assert record.id == key
-        return record
+        return SeqIO.SffIO._sff_read_seq_record(handle,
+                                                self._flows_per_read,
+                                                self._flow_chars,
+                                                self._key_sequence,
+                                                self._alphabet)
+
 
 class SffTrimmedDict(SffDict) :
     def __getitem__(self, key) :
         handle = self._handle
         handle.seek(dict.__getitem__(self, key))
-        record = SeqIO.SffIO._sff_read_seq_record(handle,
-                                                  self._flows_per_read,
-                                                  self._flow_chars,
-                                                  self._key_sequence,
-                                                  self._alphabet,
-                                                  trim=True)
-        assert record.id == key
-        return record
+        return SeqIO.SffIO._sff_read_seq_record(handle,
+                                                self._flows_per_read,
+                                                self._flow_chars,
+                                                self._key_sequence,
+                                                self._alphabet,
+                                                trim=True)
+
 
 ###################
 # Simple indexers #
 ###################
 
-class _SequentialSeqFileDict(_IndexedSeqFileDict):
-    """Subclass for easy cases (PRIVATE)."""
-    def __init__(self, filename, alphabet, key_function, format, marker):
-        _IndexedSeqFileDict.__init__(self, filename, alphabet, key_function)
-        self._format = format
+class SequentialSeqFileDict(_IndexedSeqFileDict):
+    """Indexed dictionary like access to most sequential sequence files."""
+    def __init__(self, filename, format, alphabet, key_function):
+        _IndexedSeqFileDict.__init__(self, filename, format, alphabet, key_function)
+        #Load the parser class/function once an avoid the dict lookup in each
+        #__getitem__ call:
+        i = SeqIO._FormatToIterator[format]
+        #The following alphabet code is a bit nasty... duplicates logic in
+        #Bio.SeqIO.parse()
+        if alphabet is None:
+            def _parse():
+                """Dynamically generated parser function (PRIVATE)."""
+                return i(self._handle).next()
+        else:
+            #TODO - Detect alphabet support ONCE at __init__
+            def _parse():
+                """Dynamically generated parser function (PRIVATE)."""
+                try:
+                    return i(self._handle, alphabet=alphabet).next()
+                except TypeError:
+                    return SeqIO._force_alphabet(i(self._handle),
+                                                 alphabet).next()
+        self._parse = _parse
+
+    def _build(self):
         handle = self._handle
+        marker = {"ace" : "CO ",
+                  "fasta": ">",
+                  "phd" : "BEGIN_SEQUENCE",
+                  "pir" : ">..;",
+                  "qual": ">",
+                   }[self._format]
         marker_re = re.compile("^%s" % marker)
         marker_offset = len(marker)
         self._marker_re = marker_re #saved for the get_raw method
         while True:
             offset = handle.tell()
             line = handle.readline()
-            if not line : break #End of file
             if marker_re.match(line):
                 #Here we can assume the record.id is the first word after the
                 #marker. This is generally fine... but not for GenBank, EMBL, Swiss
-                self._record_key(line[marker_offset:].strip().split(None, 1)[0], \
-                                 offset)
+                yield line[marker_offset:].strip().split(None, 1)[0], offset
+            elif not line:
+                #End of file
+                break
+
+    def __getitem__(self, key):
+        """x.__getitem__(y) <==> x[y]"""
+        handle = self._handle
+        handle.seek(dict.__getitem__(self, key))
+        record = self._parse()
+        if self._key_function:
+            assert self._key_function(record.id) == key, \
+                   "Requested key %s, found record.id %s which has key %s" \
+                   % (repr(key), repr(record.id),
+                      repr(self._key_function(record.id)))
+        else:
+            assert record.id == key, \
+                   "Requested key %s, found record.id %s" \
+                   % (repr(key), repr(record.id))
+        return record
 
     def get_raw(self, key):
         """Similar to the get method, but returns the record as a raw string."""
@@ -285,59 +327,26 @@ class _SequentialSeqFileDict(_IndexedSeqFileDict):
         data = handle.readline()
         while True:
             line = handle.readline()
-            if not line or marker_re.match(line):
+            if marker_re.match(line) or not line:
                 #End of file, or start of next record => end of this record
                 break
             data += line
         return data
 
-class FastaDict(_SequentialSeqFileDict):
-    """Indexed dictionary like access to a FASTA file."""
-    def __init__(self, filename, alphabet, key_function):
-        _SequentialSeqFileDict.__init__(self, filename, alphabet, key_function,
-                                        "fasta", ">")
-
-class QualDict(_SequentialSeqFileDict):
-    """Indexed dictionary like access to a QUAL file."""
-    def __init__(self, filename, alphabet, key_function):
-        _SequentialSeqFileDict.__init__(self, filename, alphabet, key_function,
-                                        "qual", ">")
-
-class PirDict(_SequentialSeqFileDict):
-    """Indexed dictionary like access to a PIR/NBRF file."""
-    def __init__(self, filename, alphabet, key_function):
-        _SequentialSeqFileDict.__init__(self, filename, alphabet, key_function,
-                                        "pir", ">..;")
-
-class PhdDict(_SequentialSeqFileDict):
-    """Indexed dictionary like access to a PHD (PHRED) file."""
-    def __init__(self, filename, alphabet, key_function):
-        _SequentialSeqFileDict.__init__(self, filename, alphabet, key_function,
-                                        "phd", "BEGIN_SEQUENCE")
-
-class AceDict(_SequentialSeqFileDict):
-    """Indexed dictionary like access to an ACE file."""
-    def __init__(self, filename, alphabet, key_function):
-        _SequentialSeqFileDict.__init__(self, filename, alphabet, key_function,
-                                        "ace", "CO ")
-
 
 #######################################
 # Fiddly indexers: GenBank, EMBL, ... #
 #######################################
 
-class GenBankDict(_SequentialSeqFileDict):
+class GenBankDict(SequentialSeqFileDict):
     """Indexed dictionary like access to a GenBank file."""
-    def __init__(self, filename, alphabet, key_function):
-        _IndexedSeqFileDict.__init__(self, filename, alphabet, key_function)
-        self._format = "genbank"
+    def _build(self):
         handle = self._handle
         marker_re = re.compile("^LOCUS ")
         self._marker_re = marker_re #saved for the get_raw method
         while True:
             offset = handle.tell()
             line = handle.readline()
-            if not line : break #End of file
             if marker_re.match(line):
                 #We cannot assume the record.id is the first word after LOCUS,
                 #normally the first entry on the VERSION or ACCESSION line is used.
@@ -350,7 +359,7 @@ class GenBankDict(_SequentialSeqFileDict):
                     elif line.startswith("VERSION "):
                         version_id = line.rstrip().split()[1]
                         if version_id.count(".")==1 and version_id.split(".")[1].isdigit():
-                            #This should mimics the GenBank parser...
+                            #This should mimic the GenBank parser...
                             key = version_id
                             done = True
                             break
@@ -363,20 +372,21 @@ class GenBankDict(_SequentialSeqFileDict):
                         break
                 if not key:
                     raise ValueError("Did not find ACCESSION/VERSION lines")
-                self._record_key(key, offset)
+                yield key, offset
+            elif not line:
+                #End of file
+                break
 
-class EmblDict(_SequentialSeqFileDict):
+
+class EmblDict(SequentialSeqFileDict):
     """Indexed dictionary like access to an EMBL file."""
-    def __init__(self, filename, alphabet, key_function):
-        _IndexedSeqFileDict.__init__(self, filename, alphabet, key_function)
-        self._format = "embl"
+    def _build(self):
         handle = self._handle
         marker_re = re.compile("^ID ")
         self._marker_re = marker_re #saved for the get_raw method
         while True:
             offset = handle.tell()
             line = handle.readline()
-            if not line : break #End of file
             if marker_re.match(line):
                 #We cannot assume the record.id is the first word after ID,
                 #normally the SV line is used.
@@ -406,56 +416,126 @@ class EmblDict(_SequentialSeqFileDict):
                     or marker_re.match(line) \
                     or not line:
                         break
-                self._record_key(key, offset)
+                yield key, offset
+            elif not line:
+                #End of file
+                break
 
-class SwissDict(_SequentialSeqFileDict):
+
+class SwissDict(SequentialSeqFileDict):
     """Indexed dictionary like access to a SwissProt file."""
-    def __init__(self, filename, alphabet, key_function):
-        _IndexedSeqFileDict.__init__(self, filename, alphabet, key_function)
-        self._format = "swiss"
+    def _build(self):
         handle = self._handle
         marker_re = re.compile("^ID ")
         self._marker_re = marker_re #saved for the get_raw method
         while True:
             offset = handle.tell()
             line = handle.readline()
-            if not line : break #End of file
             if marker_re.match(line):
                 #We cannot assume the record.id is the first word after ID,
                 #normally the following AC line is used.
                 line = handle.readline()
                 assert line.startswith("AC ")
                 key = line[3:].strip().split(";")[0].strip()
-                self._record_key(key, offset)
+                yield key, offset
+            elif not line:
+                #End of file
+                break
 
-class IntelliGeneticsDict(_SequentialSeqFileDict):
+
+class UniprotDict(SequentialSeqFileDict):
+    """Indexed dictionary like access to a UniProt XML file."""
+    def _build(self):
+        handle = self._handle
+        marker_re = re.compile("^<entry ") #Assumes start of line!
+        self._marker_re = marker_re #saved for the get_raw method
+        while True:
+            offset = handle.tell()
+            line = handle.readline()
+            if marker_re.match(line):
+                #We expect the next line to be <accession>xxx</accession>
+                #but allow it to be later on within the <entry>
+                key = None
+                done = False
+                while True:
+                    line = handle.readline()
+                    if line.startswith("<accession>"):
+                        assert "</accession>" in line, line
+                        key = line[11:].split("<")[0]
+                        break
+                    elif "</entry>" in line:
+                        break
+                    elif marker_re.match(line) or not line:
+                        #Start of next record or end of file
+                        raise ValueError("Didn't find end of record")
+                if not key:
+                    raise ValueError("Did not find <accession> line")
+                yield key, offset
+            elif not line:
+                #End of file
+                break
+    
+    def get_raw(self, key):
+        """Similar to the get method, but returns the record as a raw string."""
+        handle = self._handle
+        marker_re = self._marker_re
+        handle.seek(dict.__getitem__(self, key))
+        data = handle.readline()
+        while True:
+            line = handle.readline()
+            i = line.find("</entry>")
+            if i != -1:
+                data += line[:i+8]
+                break
+            if marker_re.match(line) or not line:
+                #End of file, or start of next record
+                raise ValueError("Didn't find end of record")
+            data += line
+        return data
+
+    def __getitem__(self, key) :
+        #TODO - Can we handle this directly in the parser?
+        #This is a hack - use get_raw for <entry>...</entry> and wrap it with
+        #the apparently required XML header and footer.
+        data = """<?xml version='1.0' encoding='UTF-8'?>
+        <uniprot xmlns="http://uniprot.org/uniprot"
+        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
+        xsi:schemaLocation="http://uniprot.org/uniprot
+        http://www.uniprot.org/support/docs/uniprot.xsd">
+        %s
+        </uniprot>
+        """ % self.get_raw(key)
+        #TODO - For consistency, this function should not accept a string:
+        return SeqIO.UniprotIO.UniprotIterator(data).next()
+
+
+class IntelliGeneticsDict(SequentialSeqFileDict):
     """Indexed dictionary like access to a IntelliGenetics file."""
-    def __init__(self, filename, alphabet, key_function):
-        _IndexedSeqFileDict.__init__(self, filename, alphabet, key_function)
-        self._format = "ig"
+    def _build(self):
         handle = self._handle
         marker_re = re.compile("^;")
         self._marker_re = marker_re #saved for the get_raw method
         while True:
             offset = handle.tell()
             line = handle.readline()
-            if not line : break #End of file
             if marker_re.match(line):
                 #Now look for the first line which doesn't start ";"
                 while True:
                     line = handle.readline()
-                    if not line:
-                        raise ValueError("Premature end of file?")
                     if line[0] != ";" and line.strip():
                         key = line.split()[0]
-                        self._record_key(key, offset)
+                        yield key, offset
                         break
+                    if not line:
+                        raise ValueError("Premature end of file?")
+            elif not line:
+                #End of file
+                break
 
-class TabDict(_IndexedSeqFileDict):
+
+class TabDict(SequentialSeqFileDict):
     """Indexed dictionary like access to a simple tabbed file."""
-    def __init__(self, filename, alphabet, key_function):
-        _IndexedSeqFileDict.__init__(self, filename, alphabet, key_function)
-        self._format = "tab"
+    def _build(self):
         handle = self._handle
         while True:
             offset = handle.tell()
@@ -470,7 +550,7 @@ class TabDict(_IndexedSeqFileDict):
                 else:
                     raise err
             else:
-                self._record_key(key, offset)
+                yield key, offset
 
     def get_raw(self, key):
         """Like the get method, but returns the record as a raw string."""
@@ -478,19 +558,18 @@ class TabDict(_IndexedSeqFileDict):
         handle.seek(dict.__getitem__(self, key))
         return handle.readline()
 
+
 ##########################
 # Now the FASTQ indexers #
 ##########################
          
-class _FastqSeqFileDict(_IndexedSeqFileDict):
-    """Subclass for easy cases (PRIVATE).
-
-    With FASTQ the records all start with a "@" line, but so too can some
-    quality lines. Note this will cope with line-wrapped FASTQ files.
+class FastqDict(SequentialSeqFileDict):
+    """Indexed dictionary like access to a FASTQ file (any supported variant).
+    
+    With FASTQ the records all start with a "@" line, but so can quality lines.
+    Note this will cope with line-wrapped FASTQ files.
     """
-    def __init__(self, filename, alphabet, key_function, fastq_format):
-        _IndexedSeqFileDict.__init__(self, filename, alphabet, key_function)
-        self._format = fastq_format
+    def _build(self):
         handle = self._handle
         pos = handle.tell()
         line = handle.readline()
@@ -502,7 +581,7 @@ class _FastqSeqFileDict(_IndexedSeqFileDict):
         while line:
             #assert line[0]=="@"
             #This record seems OK (so far)
-            self._record_key(line[1:].rstrip().split(None, 1)[0], pos)
+            yield line[1:].rstrip().split(None, 1)[0], pos
             #Find the seq line(s)
             seq_len = 0
             while line:
@@ -571,42 +650,27 @@ class _FastqSeqFileDict(_IndexedSeqFileDict):
             raise ValueError("Problem with quality section")
         return data
 
-class FastqSangerDict(_FastqSeqFileDict):
-    """Indexed dictionary like access to a standard Sanger FASTQ file."""
-    def __init__(self, filename, alphabet, key_function):
-        _FastqSeqFileDict.__init__(self, filename, alphabet, key_function,
-                                   "fastq-sanger")
-
-class FastqSolexaDict(_FastqSeqFileDict):
-    """Indexed dictionary like access to a Solexa (or early Illumina) FASTQ file."""
-    def __init__(self, filename, alphabet, key_function):
-        _FastqSeqFileDict.__init__(self, filename, alphabet, key_function,
-                                   "fastq-solexa")
-
-class FastqIlluminaDict(_FastqSeqFileDict):
-    """Indexed dictionary like access to a Illumina 1.3+ FASTQ file."""
-    def __init__(self, filename, alphabet, key_function):
-        _FastqSeqFileDict.__init__(self, filename, alphabet, key_function,
-                                   "fastq-illumina")
 
 ###############################################################################
 
-_FormatToIndexedDict = {"ace" : AceDict,
+_FormatToIndexedDict = {"ace" : SequentialSeqFileDict,
                         "embl" : EmblDict,
-                        "fasta" : FastaDict,
-                        "fastq" : FastqSangerDict,
-                        "fastq-sanger" : FastqSangerDict, #alias of the above
-                        "fastq-solexa" : FastqSolexaDict,
-                        "fastq-illumina" : FastqIlluminaDict,
+                        "fasta" : SequentialSeqFileDict,
+                        "fastq" : FastqDict, #Class handles all three variants
+                        "fastq-sanger" : FastqDict, #alias of the above
+                        "fastq-solexa" : FastqDict,
+                        "fastq-illumina" : FastqDict,
                         "genbank" : GenBankDict,
                         "gb" : GenBankDict, #alias of the above
                         "ig" : IntelliGeneticsDict,
-                        "phd" : PhdDict,
-                        "pir" : PirDict,
+                        "imgt" : EmblDict,
+                        "phd" : SequentialSeqFileDict,
+                        "pir" : SequentialSeqFileDict,
                         "sff" : SffDict,
                         "sff-trim" : SffTrimmedDict,
                         "swiss" : SwissDict,
                         "tab" : TabDict,
-                        "qual" : QualDict
+                        "qual" : SequentialSeqFileDict,
+                        "uniprot-xml" : UniprotDict,
                         }
 
