@@ -10,12 +10,27 @@
  */
 
 #include "Python.h"
-#include "csupport.h"
 
 
 #define _PRECISION 1000
 #define rint(x) (int)((x)*_PRECISION+0.5)
 
+
+/* Return a PyNumber as a double.
+ * Raises a TypeError if I can't do it.
+ */
+static double PyNumber_AsDouble(PyObject *py_num)
+{
+    double val;
+    PyObject *floatobj;
+
+    if((floatobj = PyNumber_Float(py_num)) == NULL)
+	return(0.0);
+    val = PyFloat_AsDouble(floatobj);
+    Py_DECREF(floatobj);
+    return val;
+}
+
 /* Functions in this module. */
 
 double calc_affine_penalty(int length, double open, double extend, 
@@ -198,7 +213,8 @@ static PyObject *cpairwise2__make_score_matrix_fast(
     char *sequenceA=NULL, *sequenceB=NULL;
     int use_sequence_cstring;
     double open_A, extend_A, open_B, extend_B;
-    int penalize_extend_when_opening, penalize_end_gaps, align_globally;
+    int penalize_extend_when_opening, penalize_end_gaps;
+    int align_globally, score_only;
 
     double first_A_gap, first_B_gap;
     double match, mismatch;
@@ -215,10 +231,10 @@ static PyObject *cpairwise2__make_score_matrix_fast(
 
     PyObject *py_retval = NULL;
 
-    if(!PyArg_ParseTuple(args, "OOOddddiii", &py_sequenceA, &py_sequenceB,
+    if(!PyArg_ParseTuple(args, "OOOddddiiii", &py_sequenceA, &py_sequenceB,
 			 &py_match_fn, &open_A, &extend_A, &open_B, &extend_B,
 			 &penalize_extend_when_opening, &penalize_end_gaps,
-			 &align_globally))
+			 &align_globally, &score_only))
 	return NULL;
     if(!PySequence_Check(py_sequenceA) || !PySequence_Check(py_sequenceB)) {
 	PyErr_SetString(PyExc_TypeError, 
@@ -458,6 +474,8 @@ static PyObject *cpairwise2__make_score_matrix_fast(
 		goto _cleanup_make_score_matrix_fast;
 	    PyList_SET_ITEM(py_score_row, col, py_score);
 
+	    if(score_only)
+		continue;
 	    /* Set py_trace_matrix[row][col] to a list of indexes.  On
 	       the edges of the matrix (row or column is 0), the
 	       matrix should be [None]. */
