@@ -24,6 +24,8 @@ from Bio.SeqUtils.CheckSum import crc64
 from Bio import Entrez
 from Bio.Seq import UnknownSeq
 
+from Bio._py3k import _is_int_or_long
+
 class DatabaseLoader:
     """Object used to load SeqRecord objects into a BioSQL database."""
     def __init__(self, adaptor, dbid, fetch_NCBI_taxonomy=False):
@@ -466,7 +468,7 @@ class DatabaseLoader:
         if len(taxonomic_lineage) > 1:
             #Use recursion to find out the taxon id (database key) of the parent.
             parent_taxon_id = self._get_taxon_id_from_ncbi_lineage(taxonomic_lineage[:-1])
-            assert isinstance(parent_taxon_id, int) or isinstance(parent_taxon_id, long), repr(parent_taxon_id)
+            assert _is_int_or_long(parent_taxon_id), repr(parent_taxon_id)
         else:
             parent_taxon_id = None
 
@@ -651,7 +653,7 @@ class DatabaseLoader:
                    " VALUES (%s, %s, %s, %s)"
         tag_ontology_id = self._get_ontology_id('Annotation Tags')
         for key, value in record.annotations.iteritems():
-            if key in ["references", "comment", "ncbi_taxid"]:
+            if key in ["references", "comment", "ncbi_taxid", "date"]:
                 #Handled separately
                 continue
             term_id = self._get_term_id(key, ontology_id=tag_ontology_id)
@@ -868,7 +870,7 @@ class DatabaseLoader:
             {key : [value1, value2]}
         """
         tag_ontology_id = self._get_ontology_id('Annotation Tags')
-        for qualifier_key in qualifiers.keys():
+        for qualifier_key in qualifiers:
             # Treat db_xref qualifiers differently to sequence annotation
             # qualifiers by populating the seqfeature_dbxref and dbxref
             # tables.  Other qualifiers go into the seqfeature_qualifier_value
