@@ -8,6 +8,7 @@ import os
 import sys
 import unittest
 import subprocess
+from StringIO import StringIO
 
 from Bio.Emboss.Applications import WaterCommandline, NeedleCommandline
 from Bio.Emboss.Applications import SeqretCommandline, SeqmatchallCommandline
@@ -21,29 +22,30 @@ from Bio.SeqRecord import SeqRecord
 
 #################################################################
 
+#Try to avoid problems when the OS is in another language
+os.environ['LANG'] = 'C'
+
 exes_wanted = ["water", "needle", "seqret", "transeq", "seqmatchall",
                "embossversion"]
 exes = dict() #Dictionary mapping from names to exe locations
-if sys.platform=="win32":
-    #The default installation path is C:\mEMBOSS which contains the exes.
+
+if "EMBOSS_ROOT" in os.environ:
+    #Windows default installation path is C:\mEMBOSS which contains the exes.
     #EMBOSS also sets an environment variable which we will check for.
-    try:
-        path = os.environ["EMBOSS_ROOT"]
-    except KeyError:
-        #print >> sys.stderr, "Missing EMBOSS_ROOT environment variable!"
-        raise MissingExternalDependencyError(\
-            "Install EMBOSS if you want to use Bio.Emboss.")
+    path = os.environ["EMBOSS_ROOT"]
     if os.path.isdir(path):
         for name in exes_wanted:
             if os.path.isfile(os.path.join(path, name+".exe")):
                 exes[name] = os.path.join(path, name+".exe")
     del path, name
-else:
+if sys.platform!="win32":
     import commands
     for name in exes_wanted:
         #This will "just work" if installed on the path as normal on Unix
-        if "not found" not in commands.getoutput("%s -help" % name):
+        output = commands.getoutput("%s -help" % name)
+        if "not found" not in output and "not recognized" not in output:
             exes[name] = name
+        del output
     del name
 
 if len(exes) < len(exes_wanted):
@@ -57,6 +59,7 @@ def get_emboss_version():
     child = subprocess.Popen(exes["embossversion"],
                              stdout=subprocess.PIPE,
                              stderr=subprocess.STDOUT,
+                             universal_newlines=True,
                              shell=(sys.platform!="win32"))
     stdout, stderr = child.communicate()
     assert stderr is None #Send to stdout instead
@@ -72,8 +75,12 @@ def get_emboss_version():
             #which reports 6.2.0.1 - for this return (6,2,0)
             return tuple(int(v) for v in line.strip().split("."))[:3]
         else:
-            raise ValueError(stdout)
-
+            #Either we can't understand the output, or this is really
+            #an error message not caught earlier (e.g. not in English)
+            raise MissingExternalDependencyError(\
+                "Install EMBOSS if you want to use Bio.Emboss (%s)." \
+                % line)
+            
 #To avoid confusing known errors from old versions of EMBOSS ...
 if get_emboss_version() < (6,1,0):
     raise MissingExternalDependencyError(\
@@ -98,6 +105,7 @@ def emboss_convert(filename, old_format, new_format):
                              stdin=subprocess.PIPE,
                              stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE,
+                             universal_newlines=True,
                              shell=(sys.platform!="win32"))
     child.stdin.close()
     return child.stdout
@@ -117,6 +125,7 @@ def emboss_piped_SeqIO_convert(records, old_format, new_format):
                              stdin=subprocess.PIPE,
                              stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE,
+                             universal_newlines=True,
                              shell=(sys.platform!="win32"))
     SeqIO.write(records, child.stdin, old_format)
     child.stdin.close()
@@ -137,6 +146,7 @@ def emboss_piped_AlignIO_convert(alignments, old_format, new_format):
                              stdin=subprocess.PIPE,
                              stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE,
+                             universal_newlines=True,
                              shell=(sys.platform!="win32"))
     AlignIO.write(alignments, child.stdin, old_format)
     child.stdin.close()
@@ -204,7 +214,7 @@ class SeqRetSeqIOTests(unittest.TestCase):
                 continue
             new_records = list(emboss_piped_SeqIO_convert(records, temp_format, "fasta"))
             try:
-                self.assert_(compare_records(records, new_records))
+                self.assertTrue(compare_records(records, new_records))
             except ValueError, err:
                 raise ValueError("Disagree on file %s %s in %s format: %s" \
                                  % (in_format, in_filename, temp_format, err))
@@ -213,7 +223,7 @@ class SeqRetSeqIOTests(unittest.TestCase):
                               skip_formats=[]):
         """Can Bio.SeqIO read seqret's conversion of the file?"""
         #TODO: Why can't we read EMBOSS's swiss output?
-        self.assert_(os.path.isfile(filename))
+        self.assertTrue(os.path.isfile(filename))
         old_records = list(SeqIO.parse(open(filename), old_format))
         for new_format in ["genbank","fasta","pir","embl", "ig"]:
             if new_format in skip_formats:
@@ -221,7 +231,7 @@ class SeqRetSeqIOTests(unittest.TestCase):
             handle = emboss_convert(filename, old_format, new_format)
             new_records = list(SeqIO.parse(handle, new_format))
             try:
-                self.assert_(compare_records(old_records, new_records))
+                self.assertTrue(compare_records(old_records, new_records))
             except ValueError, err:
                 raise ValueError("Disagree on %s file %s in %s format: %s" \
                                  % (old_format, filename, new_format, err))
@@ -283,7 +293,7 @@ class SeqRetAlignIOTests(unittest.TestCase):
     def check_EMBOSS_to_AlignIO(self, filename, old_format,
                               skip_formats=[]):
         """Can AlignIO read seqret's conversion of the file?"""
-        self.assert_(os.path.isfile(filename), filename)
+        self.assertTrue(os.path.isfile(filename), filename)
         old_aligns = list(AlignIO.parse(open(filename), old_format))
         formats = ["clustal", "phylip", "ig"]
         if len(old_aligns) == 1:
@@ -298,7 +308,7 @@ class SeqRetAlignIOTests(unittest.TestCase):
                 raise ValueError("Can't parse %s file %s in %s format." \
                                  % (old_format, filename, new_format))
             try:
-                self.assert_(compare_alignments(old_aligns, new_aligns))
+                self.assertTrue(compare_alignments(old_aligns, new_aligns))
             except ValueError, err:
                 raise ValueError("Disagree on %s file %s in %s format: %s" \
                                  % (old_format, filename, new_format, err))
@@ -329,7 +339,7 @@ class SeqRetAlignIOTests(unittest.TestCase):
                 #from writing Nexus files...
                 continue
             try:
-                self.assert_(compare_alignments(old_aligns, new_aligns))
+                self.assertTrue(compare_alignments(old_aligns, new_aligns))
             except ValueError, err:
                 raise ValueError("Disagree on file %s %s in %s format: %s" \
                                  % (in_format, in_filename, temp_format, err))
@@ -385,9 +395,9 @@ class PairwiseAlignmentTests(unittest.TestCase):
                                      % (alignment[1].id , target.id, target.name))
             if local:
                 #Local alignment
-                self.assert_(str(alignment[0].seq).replace("-","") \
+                self.assertTrue(str(alignment[0].seq).replace("-","") \
                              in query_seq)
-                self.assert_(str(alignment[1].seq).replace("-","").upper() \
+                self.assertTrue(str(alignment[1].seq).replace("-","").upper() \
                              in str(target.seq).upper())
             else:
                 #Global alignment
@@ -402,17 +412,18 @@ class PairwiseAlignmentTests(unittest.TestCase):
                                  stdin=subprocess.PIPE,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE,
+                                 universal_newlines=True,
                                  shell=(sys.platform!="win32"))
         #Check it worked,
         return_code = child.wait()
         if return_code != 0 : print >> sys.stderr, "\n%s"%cline
         self.assertEqual(return_code, 0)
         errors = child.stderr.read().strip()
-        self.assert_(errors.startswith("Smith-Waterman local alignment"),
+        self.assertTrue(errors.startswith("Smith-Waterman local alignment"),
                      errors)
         if cline.outfile:
             self.assertEqual(child.stdout.read().strip(), "")
-            self.assert_(os.path.isfile(cline.outfile))
+            self.assertTrue(os.path.isfile(cline.outfile))
         else :
             #Don't use this yet... could return stdout handle instead?
             return child.stdout.read()
@@ -431,7 +442,7 @@ class PairwiseAlignmentTests(unittest.TestCase):
         #Run the tool,
         self.run_water(cline)
         #Check we can parse the output...
-        align = AlignIO.read(open(cline.outfile),"emboss")
+        align = AlignIO.read(cline.outfile,"emboss")
         self.assertEqual(len(align), 2)
         self.assertEqual(str(align[0].seq), "ACCCGGGCGCGGT")
         self.assertEqual(str(align[1].seq), "ACCCGAGCGCGGT")
@@ -456,6 +467,7 @@ class PairwiseAlignmentTests(unittest.TestCase):
                                  stdin=subprocess.PIPE,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE,
+                                 universal_newlines=True,
                                  shell=(sys.platform!="win32"))
         child.stdin.close()
         #Check we could read it's output
@@ -464,8 +476,8 @@ class PairwiseAlignmentTests(unittest.TestCase):
         self.assertEqual(str(align[0].seq), "ACCCGGGCGCGGT")
         self.assertEqual(str(align[1].seq), "ACCCGAGCGCGGT")
         #Check no error output:
-        assert child.stderr.read() == ""
-        assert 0 == child.wait()
+        self.assertEqual(child.stderr.read(), "")
+        self.assertEqual(0, child.wait())
 
     def test_needle_file(self):
         """needle with the asis trick, output to a file."""
@@ -484,19 +496,20 @@ class PairwiseAlignmentTests(unittest.TestCase):
                                  stdin=subprocess.PIPE,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE,
+                                 universal_newlines=True,
                                  shell=(sys.platform!="win32"))
         out, err = child.communicate()
         return_code = child.returncode
         #Check it worked,
         errors = err.strip()
-        self.assert_(err.strip().startswith("Needleman-Wunsch global alignment"), errors)
+        self.assertTrue(err.strip().startswith("Needleman-Wunsch global alignment"), errors)
         self.assertEqual(out.strip(), "")
         if return_code != 0 : print >> sys.stderr, "\n%s"%cline
         self.assertEqual(return_code, 0)
         filename = cline.outfile
-        self.assert_(os.path.isfile(filename))
+        self.assertTrue(os.path.isfile(filename))
         #Check we can parse the output...
-        align = AlignIO.read(open(filename),"emboss")
+        align = AlignIO.read(filename,"emboss")
         self.assertEqual(len(align), 2)
         self.assertEqual(str(align[0].seq), "ACCCGGGCGCGGT")
         self.assertEqual(str(align[1].seq), "ACCCGAGCGCGGT")
@@ -521,6 +534,7 @@ class PairwiseAlignmentTests(unittest.TestCase):
                                  stdin=subprocess.PIPE,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE,
+                                 universal_newlines=True,
                                  shell=(sys.platform!="win32"))
         child.stdin.close()
         #Check we could read it's output
@@ -529,8 +543,8 @@ class PairwiseAlignmentTests(unittest.TestCase):
         self.assertEqual(str(align[0].seq), "ACCCGGGCGCGGT")
         self.assertEqual(str(align[1].seq), "ACCCGAGCGCGGT")
         #Check no error output:
-        assert child.stderr.read() == ""
-        assert 0 == child.wait()
+        self.assertEqual(child.stderr.read(), "")
+        self.assertEqual(0, child.wait())
 
     def test_water_file2(self):
         """water with the asis trick and nucleotide FASTA file, output to a file."""
@@ -538,7 +552,7 @@ class PairwiseAlignmentTests(unittest.TestCase):
         query = "ACACACTCACACACACTTGGTCAGAGATGCTGTGCTTCTTGGAAGCAAGGNCTCAAAGGCAAGGTGCACGCAGAGGGACGTTTGAGTCTGGGATGAAGCATGTNCGTATTATTTATATGATGGAATTTCACGTTTTTATG"
         out_file = "Emboss/temp_test2.water"
         in_file = "Fasta/f002"
-        self.assert_(os.path.isfile(in_file))
+        self.assertTrue(os.path.isfile(in_file))
         if os.path.isfile(out_file):
             os.remove(out_file)
         cline = WaterCommandline(cmd=exes["water"])
@@ -552,8 +566,8 @@ class PairwiseAlignmentTests(unittest.TestCase):
         self.run_water(cline)
         #Check we can parse the output and it is sensible...
         self.pairwise_alignment_check(query,
-                                      SeqIO.parse(open(in_file),"fasta"),
-                                      AlignIO.parse(open(out_file),"emboss"),
+                                      SeqIO.parse(in_file,"fasta"),
+                                      AlignIO.parse(out_file,"emboss"),
                                       local=True)
         #Clean up,
         os.remove(out_file)
@@ -564,7 +578,7 @@ class PairwiseAlignmentTests(unittest.TestCase):
         query = "TGTTGTAATGTTTTAATGTTTCTTCTCCCTTTAGATGTACTACGTTTGGA"
         out_file = "Emboss/temp_test3.water"
         in_file = "GenBank/cor6_6.gb"
-        self.assert_(os.path.isfile(in_file))
+        self.assertTrue(os.path.isfile(in_file))
         if os.path.isfile(out_file):
             os.remove(out_file)
         cline = WaterCommandline(cmd=exes["water"])
@@ -579,8 +593,8 @@ class PairwiseAlignmentTests(unittest.TestCase):
         self.run_water(cline)
         #Check we can parse the output and it is sensible...
         self.pairwise_alignment_check(query,
-                                      SeqIO.parse(open(in_file),"genbank"),
-                                      AlignIO.parse(open(out_file),"emboss"),
+                                      SeqIO.parse(in_file,"genbank"),
+                                      AlignIO.parse(out_file,"emboss"),
                                       local=True)
         #Clean up,
         os.remove(out_file)
@@ -591,7 +605,7 @@ class PairwiseAlignmentTests(unittest.TestCase):
         query = "DVCTGKALCDPVTQNIKTYPVKIENLRVMI"
         out_file = "Emboss/temp_test4.water"
         in_file = "SwissProt/sp004"
-        self.assert_(os.path.isfile(in_file))
+        self.assertTrue(os.path.isfile(in_file))
         if os.path.isfile(out_file):
             os.remove(out_file)
         cline = WaterCommandline(cmd=exes["water"])
@@ -608,8 +622,8 @@ class PairwiseAlignmentTests(unittest.TestCase):
         self.run_water(cline)
         #Check we can parse the output and it is sensible...
         self.pairwise_alignment_check(query,
-                                      SeqIO.parse(open(in_file),"swiss"),
-                                      AlignIO.parse(open(out_file),"emboss"),
+                                      SeqIO.parse(in_file,"swiss"),
+                                      AlignIO.parse(out_file,"emboss"),
                                       local=True)
         #Clean up,
         os.remove(out_file)
@@ -630,6 +644,7 @@ class PairwiseAlignmentTests(unittest.TestCase):
                                  stdin=subprocess.PIPE,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE,
+                                 universal_newlines=True,
                                  shell=(sys.platform!="win32"))
         child.stdin.close()
         #Check we can parse the output and it is sensible...
@@ -638,8 +653,8 @@ class PairwiseAlignmentTests(unittest.TestCase):
                                       AlignIO.parse(child.stdout,"emboss"),
                                       local=False)
         #Check no error output:
-        assert child.stderr.read() == ""
-        assert 0 == child.wait()
+        self.assertEqual(child.stderr.read(), "")
+        self.assertEqual(0, child.wait())
 
     def test_water_needs_output(self):
         """water without output file or stdout/filter should give error."""
@@ -649,9 +664,9 @@ class PairwiseAlignmentTests(unittest.TestCase):
                                  gapopen=10,
                                  gapextend=0.5,
                                  auto=True)
-        self.assert_(cline.auto)
-        self.assert_(not cline.stdout)
-        self.assert_(not cline.filter)
+        self.assertTrue(cline.auto)
+        self.assertTrue(not cline.stdout)
+        self.assertTrue(not cline.filter)
         self.assertEqual(cline.outfile, None)
         self.assertRaises(ValueError, str, cline)
 
@@ -663,9 +678,9 @@ class PairwiseAlignmentTests(unittest.TestCase):
                                  gapopen=10,
                                  gapextend=0.5,
                                  auto=True)
-        self.assert_(cline.auto)
-        self.assert_(not cline.stdout)
-        self.assert_(not cline.filter)
+        self.assertTrue(cline.auto)
+        self.assertTrue(not cline.stdout)
+        self.assertTrue(not cline.filter)
         self.assertEqual(cline.outfile, None)
         self.assertRaises(ValueError, str, cline)
    
@@ -684,6 +699,7 @@ class PairwiseAlignmentTests(unittest.TestCase):
                                  stdin=subprocess.PIPE,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE,
+                                 universal_newlines=True,
                                  shell=(sys.platform!="win32"))
         child.stdin.close()
         #Check we could read it's output
@@ -691,8 +707,8 @@ class PairwiseAlignmentTests(unittest.TestCase):
             self.assertEqual(len(align), 2)
             self.assertEqual(align.get_alignment_length(), 9)
         #Check no error output:
-        assert child.stderr.read() == ""
-        assert 0 == child.wait()
+        self.assertEqual(child.stderr.read(), "")
+        self.assertEqual(0, child.wait())
         
 #Top level function as this makes it easier to use for debugging:
 def emboss_translate(sequence, table=None, frame=None):
@@ -713,10 +729,7 @@ def emboss_translate(sequence, table=None, frame=None):
         #There are limits on command line string lengths...
         #use a temp file instead.
         filename = "Emboss/temp_transeq.txt"
-        handle = open(filename,"w")
-        SeqIO.write([SeqRecord(sequence, id="Test")], handle, "fasta")
-        handle.flush()
-        handle.close()
+        SeqIO.write(SeqRecord(sequence, id="Test"), filename, "fasta")
         cline += " -sequence %s" % filename
 
     cline += " -auto" #no prompting
@@ -730,15 +743,15 @@ def emboss_translate(sequence, table=None, frame=None):
                              stdin=subprocess.PIPE,
                              stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE,
+                             universal_newlines=True,
                              shell=(sys.platform!="win32"))
-    child.stdin.close()
+    out, err = child.communicate()
     #Check no error output:
-    err = child.stderr.read()
     if err != "":
         raise ValueError(str(cline) + "\n" + err)
 
     #Check we could read it's output
-    record = SeqIO.read(child.stdout, "fasta")
+    record = SeqIO.read(StringIO(out), "fasta")
 
     if 0 != child.wait():
         raise ValueError(str(cline))
@@ -810,7 +823,7 @@ class TranslationTests(unittest.TestCase):
             if len(sequence) % 3 != 0:
                 sequence = sequence[:-(len(sequence)%3)]
             self.assertEqual(len(sequence) % 3, 0)
-            self.assert_(len(sequence) > 0)
+            self.assertTrue(len(sequence) > 0)
             self.check(sequence)
 
     def check(self, sequence):
@@ -818,11 +831,11 @@ class TranslationTests(unittest.TestCase):
 
         Takes a Seq object (and a filename containing it)."""
         translation = emboss_translate(sequence)
-        self.assert_(check_translation(sequence, translation))
+        self.assertTrue(check_translation(sequence, translation))
 
         for table in [1,2,3,4,5,6,9,10,11,12,13,14,15,16,21,22,23]:
             translation = emboss_translate(sequence, table)
-            self.assert_(check_translation(sequence, translation, table))
+            self.assertTrue(check_translation(sequence, translation, table))
         return True
 
     def translate_all_codons(self, letters):
