@@ -201,7 +201,7 @@ class PrimersearchParseTest(unittest.TestCase):
         h.close()
 
         self.assertEqual(len(amp_info.amplifiers), 1)
-        self.assert_("Test" in amp_info.amplifiers)
+        self.assertTrue("Test" in amp_info.amplifiers)
         self.assertEqual(len(amp_info.amplifiers["Test"]), 1)
 
         self.assertEqual(amp_info.amplifiers["Test"][0].length, 218)
