@@ -3,11 +3,12 @@
 """
 
 import warnings
-warnings.filterwarnings("ignore", category=DeprecationWarning)
+from Bio import BiopythonDeprecationWarning
+warnings.filterwarnings("ignore", category=BiopythonDeprecationWarning)
 import Bio.GFF
 import Bio.GFF.GenericTools
 import Bio.GFF.easy
-warnings.resetwarnings()
+warnings.filters.pop()
 
 print "Running Bio.GFF.GenericTools doctests..."
 Bio.GFF.GenericTools._test()
