@@ -45,6 +45,7 @@ for folder in likely_dirs:
         child = subprocess.Popen(exe_name + " -h",
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE,
+                                 universal_newlines=True,
                                  shell=(sys.platform!="win32"))
         output, error = child.communicate()
         if child.returncode==0 and "ERROR: Invalid argument: -h" not in output:
@@ -73,6 +74,7 @@ class Pairwise(unittest.TestCase):
         child = subprocess.Popen(str(cline),
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE,
+                                 universal_newlines=True,
                                  shell=(sys.platform!="win32"))
         stdoutdata, stderrdata = child.communicate()
         return_code = child.returncode
@@ -88,7 +90,7 @@ class Pairwise(unittest.TestCase):
     def test_blastn(self):
         """Pairwise BLASTN search"""
         global exe_names
-        cline = Applications.NcbiblastpCommandline(exe_names["blastn"],
+        cline = Applications.NcbiblastnCommandline(exe_names["blastn"],
                         query="GenBank/NC_005816.ffn",
                         subject="GenBank/NC_005816.fna",
                         evalue="0.000001")
@@ -98,6 +100,7 @@ class Pairwise(unittest.TestCase):
         child = subprocess.Popen(str(cline),
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE,
+                                 universal_newlines=True,
                                  shell=(sys.platform!="win32"))
         stdoutdata, stderrdata = child.communicate()
         return_code = child.returncode
@@ -110,7 +113,7 @@ class Pairwise(unittest.TestCase):
     def test_tblastn(self):
         """Pairwise TBLASTN search"""
         global exe_names
-        cline = Applications.NcbiblastpCommandline(exe_names["tblastn"],
+        cline = Applications.NcbitblastnCommandline(exe_names["tblastn"],
                         query="GenBank/NC_005816.faa",
                         subject="GenBank/NC_005816.fna",
                         evalue="1e-6")
@@ -120,6 +123,7 @@ class Pairwise(unittest.TestCase):
         child = subprocess.Popen(str(cline),
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE,
+                                 universal_newlines=True,
                                  shell=(sys.platform!="win32"))
         stdoutdata, stderrdata = child.communicate()
         return_code = child.returncode
@@ -142,6 +146,7 @@ class CheckCompleteArgList(unittest.TestCase):
         child = subprocess.Popen(str(cline),
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE,
+                                 universal_newlines=True,
                                  shell=(sys.platform!="win32"))
         stdoutdata, stderrdata = child.communicate()
         self.assertEqual(stderrdata, "",
@@ -175,14 +180,35 @@ class CheckCompleteArgList(unittest.TestCase):
         if "-use_test_remote_service" in missing :
             #Known issue, seems to be present in some builds (Bug 3043)
             missing.remove("-use_test_remote_service")
+        if exe_name == "blastn" and "-off_diagonal_range" in extra:
+            #Added in BLAST 2.2.23+
+            extra.remove("-off_diagonal_range")
+        if exe_name == "tblastx":
+            #These appear to have been removed in BLAST 2.2.23+
+            #(which seems a bit odd - TODO - check with NCBI?)
+            extra = extra.difference(["-gapextend","-gapopen",
+                                      "-xdrop_gap","-xdrop_gap_final"])
+        if exe_name in ["rpsblast", "rpstblastn"]:
+            #These appear to have been removed in BLAST 2.2.24+
+            #(which seems a bit odd - TODO - check with NCBI?)
+            extra = extra.difference(["-num_threads"])
+        if exe_name in ["tblastn", "tblastx"]:
+            #These appear to have been removed in BLAST 2.2.24+
+            extra = extra.difference(["-db_soft_mask"])
+        #This was added in BLAST 2.2.24+ to most/all the tools, so
+        #will be seen as an extra argument on older versions:
+        if "-seqidlist" in extra:
+            extra.remove("-seqidlist")
 
-        if extra or missing :
-            raise MissingExternalDependencyError("BLAST+ and Biopython out of sync. "
-                  "Your version of the NCBI BLAST+ tool %s does not match what we "
-                  "are expecting. Please update your copy of Biopython, or report "
-                  "this issue if you are already using the latest version. "
-                  "(Exta args: %s; Missing: %s)" \
-                  % (exe_name, ",".join(sorted(extra)), ",".join(sorted(missing))))
+        if extra or missing:
+            raise MissingExternalDependencyError("BLAST+ and Biopython out "
+                  "of sync. Your version of the NCBI BLAST+ tool %s does not "
+                  "match what we are expecting. Please update your copy of "
+                  "Biopython, or report this issue if you are already using "
+                  "the latest version. (Exta args: %s; Missing: %s)" \
+                  % (exe_name,
+                     ",".join(sorted(extra)),
+                     ",".join(sorted(missing))))
 
         #An almost trivial example to test any validation
         cline = wrapper(exe, query="dummy")
