@@ -3,7 +3,13 @@
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
 
-"""Additional unit tests for Bio.SeqIO.convert(...) function."""
+"""Additional unit tests for Bio.SeqIO.index(...) function."""
+import sys
+if sys.version_info[0] >= 3:
+    from Bio import MissingExternalDependencyError
+    raise MissingExternalDependencyError(\
+        "Skipping since currently this is very slow on Python 3.")
+    
 import os
 import unittest
 from StringIO import StringIO
@@ -14,6 +20,10 @@ from Bio.Alphabet import generic_protein, generic_nucleotide, generic_dna
 
 from seq_tests_common import compare_record
 
+def add_prefix(key):
+    """Dummy key_function for testing index code."""
+    return "id_" + key
+
 class IndexDictTests(unittest.TestCase):
     """Cunning unit test where methods are added at run time."""
     def simple_check(self, filename, format, alphabet):
@@ -23,17 +33,27 @@ class IndexDictTests(unittest.TestCase):
             mode = "r"
         id_list = [rec.id for rec in \
                    SeqIO.parse(open(filename, mode), format, alphabet)]
+        #Without key_function
         rec_dict = SeqIO.index(filename, format, alphabet)
-        self.assertEqual(set(id_list), set(rec_dict.keys()))
+        self.check_dict_methods(rec_dict, id_list, id_list)
+        #Check with key_function
+        key_list = [add_prefix(id) for id in id_list]
+        rec_dict = SeqIO.index(filename, format, alphabet, add_prefix)
+        self.check_dict_methods(rec_dict, key_list, id_list)
+        #Done
+    
+    def check_dict_methods(self, rec_dict, keys, ids):
+        self.assertEqual(set(keys), set(rec_dict.keys()))
         #This is redundant, I just want to make sure len works:
-        self.assertEqual(len(id_list), len(rec_dict))
+        self.assertEqual(len(keys), len(rec_dict))
         #Make sure boolean evaluation works
-        self.assertEqual(bool(id_list), bool(rec_dict))
-        for key in id_list:
-            self.assert_(key in rec_dict)
-            self.assertEqual(key, rec_dict[key].id)
-            self.assertEqual(key, rec_dict.get(key).id)
+        self.assertEqual(bool(keys), bool(rec_dict))
+        for key,id in zip(keys, ids):
+            self.assertTrue(key in rec_dict)
+            self.assertEqual(id, rec_dict[key].id)
+            self.assertEqual(id, rec_dict.get(key).id)
         #Check non-existant keys,
+        assert chr(0) not in keys, "Bad example in test"
         try:
             rec = rec_dict[chr(0)]
             raise ValueError("Accessing a non-existent key should fail")
@@ -41,13 +61,27 @@ class IndexDictTests(unittest.TestCase):
             pass
         self.assertEqual(rec_dict.get(chr(0)), None)
         self.assertEqual(rec_dict.get(chr(0), chr(1)), chr(1))
-        #Now check iteritems...
-        for key, rec in rec_dict.iteritems():
-            self.assert_(key in id_list)
-            self.assert_(isinstance(rec, SeqRecord))
-            self.assertEqual(rec.id, key)
-        #Now check non-defined methods...
-        self.assertRaises(NotImplementedError, rec_dict.values)
+        if hasattr(dict, "iteritems"):
+            #Python 2.x
+            for key, rec in rec_dict.iteritems():
+                self.assertTrue(key in keys)
+                self.assertTrue(isinstance(rec, SeqRecord))
+                self.assertTrue(rec.id in ids)
+            #Now check non-defined methods...
+            self.assertRaises(NotImplementedError, rec_dict.items)
+            self.assertRaises(NotImplementedError, rec_dict.values)
+        else:
+            #Python 3
+            assert not hasattr(rec_dict, "iteritems")
+            for key, rec in rec_dict.iteritems():
+                self.assertTrue(key in id_list)
+                self.assertTrue(isinstance(rec, SeqRecord))
+                self.assertTrue(rec.id in ids)
+            for rec in rec_dict.itervalues():
+                self.assertTrue(key in id_list)
+                self.assertTrue(isinstance(rec, SeqRecord))
+                self.assertTrue(rec.id in ids)
+        #Check the following fail
         self.assertRaises(NotImplementedError, rec_dict.popitem)
         self.assertRaises(NotImplementedError, rec_dict.pop, chr(0))
         self.assertRaises(NotImplementedError, rec_dict.pop, chr(0), chr(1))
@@ -55,7 +89,6 @@ class IndexDictTests(unittest.TestCase):
         self.assertRaises(NotImplementedError, rec_dict.__setitem__, "X", None)
         self.assertRaises(NotImplementedError, rec_dict.copy)
         self.assertRaises(NotImplementedError, rec_dict.fromkeys, [])
-        #Done
 
     def get_raw_check(self, filename, format, alphabet):
         if format in SeqIO._BinaryFormats:
@@ -73,21 +106,31 @@ class IndexDictTests(unittest.TestCase):
         self.assertEqual(set(id_list), set(rec_dict.keys()))
         self.assertEqual(len(id_list), len(rec_dict))
         for key in id_list:
-            self.assert_(key in rec_dict)
+            self.assertTrue(key in rec_dict)
             self.assertEqual(key, rec_dict[key].id.lower())
             self.assertEqual(key, rec_dict.get(key).id.lower())
             raw = rec_dict.get_raw(key)
-            self.assert_(raw.strip())
-            self.assert_(raw in raw_file)
-            if format in ["ig"]:
+            self.assertTrue(raw.strip())
+            self.assertTrue(raw in raw_file)
+            if format in ["ig", "uniprot-xml"]:
                #These have a header structure and can't be parsed
                #individually (at least, not right now).
                continue
             rec1 = rec_dict[key]
             rec2 = SeqIO.read(StringIO(raw), format, alphabet)
-	    self.assertEqual(True, compare_record(rec1, rec2))
-        #Done
-            
+            self.assertEqual(True, compare_record(rec1, rec2))
+
+    def test_duplicates_index(self):
+        """Index file with duplicate identifers with Bio.SeqIO.index()"""
+        self.assertRaises(ValueError, SeqIO.index, "Fasta/dups.fasta", "fasta")
+
+    def test_duplicates_to_dict(self):
+        """Index file with duplicate identifers with Bio.SeqIO.to_dict()"""
+        handle = open("Fasta/dups.fasta", "rU")
+        iterator = SeqIO.parse(handle, "fasta")
+        self.assertRaises(ValueError, SeqIO.to_dict, iterator)
+        handle.close()
+
 tests = [
     ("Ace/contig1.ace", "ace", generic_dna),
     ("Ace/consed_sample.ace", "ace", None),
@@ -99,9 +142,11 @@ tests = [
     ("Quality/sanger_faked.fastq", "fastq-sanger", generic_dna),
     ("Quality/solexa_faked.fastq", "fastq-solexa", generic_dna),
     ("Quality/illumina_faked.fastq", "fastq-illumina", generic_dna),
+    ("EMBL/epo_prt_selection.embl", "embl", None),
     ("EMBL/U87107.embl", "embl", None),
     ("EMBL/TRBG361.embl", "embl", None),
     ("EMBL/A04195.imgt", "embl", None), #Not a proper EMBL file, an IMGT file
+    ("EMBL/A04195.imgt", "imgt", None),
     ("GenBank/NC_000932.faa", "fasta", generic_protein),
     ("GenBank/NC_005816.faa", "fasta", generic_protein),
     ("GenBank/NC_005816.tsv", "tab", generic_protein),
@@ -122,6 +167,9 @@ tests = [
     ("SwissProt/sp001", "swiss", None),
     ("SwissProt/sp010", "swiss", None),
     ("SwissProt/sp016", "swiss", None),
+    ("SwissProt/multi_ex.txt", "swiss", None),
+    ("SwissProt/multi_ex.xml", "uniprot-xml", None),
+    ("SwissProt/multi_ex.fasta", "fasta", None),
     ("Roche/E3MFGYR02_random_10_reads.sff", "sff", generic_dna),
     ("Roche/E3MFGYR02_random_10_reads.sff", "sff-trim", generic_dna),
     ("Roche/E3MFGYR02_index_at_start.sff", "sff", generic_dna),
@@ -137,6 +185,12 @@ tests = [
     ]
 for filename, format, alphabet in tests:
     assert format in _FormatToIndexedDict
+
+    #TODO - remove this hack once we drop Python 2.4
+    if format=="uniprot-xml" and SeqIO.UniprotIO.ElementTree is None:
+        #skip this test
+        continue
+    
     def funct(fn,fmt,alpha):
         f = lambda x : x.simple_check(fn, fmt, alpha)
         f.__doc__ = "Index %s file %s" % (fmt, fn)
